/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef __FUNAMBOLAPPVIEW_h__
#define __FUNAMBOLAPPVIEW_h__


#include <coecntrl.h>
#include <aknsbasicbackgroundcontrolcontext.h>
#include "FunambolAppUi.h"

#include <aknlists.h> 	// For List Box
#include <barsread.h> 	// For Resource Reader
#include <eikclbd.h>    // For iListBox->ItemDrawer() !


typedef struct ListboxItem {
    TInt        sourceId;       // The corresponding source ID
    TBuf<32>    label;          // The label displayed (first line)
    TBuf<32>    status;         // The status displayed (second line)
    bool        hidden;         // If true, the panel is hidden
} ListboxItems;



class CFunambolAppView : public CCoeControl
{
    
private:
    
    CFunambolAppUi*                     iAppUI;
    
    /// The graphical listbox (icon + label + smaller text).
    CEikFormattedCellListBoxTypedef*    iListBox;
    
    /// The listbox struct of data, is dynamically filled at startup.
    ListboxItems*                       iListboxItems;
    
    /// The total number of items in the listbox.
    /// DO NOT CHANGE THIS VALUE! It's initialized only once in LoadListboxData().
    TInt                                iListboxNumberOfItems;
    
    CAknsBasicBackgroundControlContext* iBgContext;
    
    /// Used to avoid the cuncurrent refresh of Listbox items, from different threads.
    RCriticalSection                    iCriticalListboxText;


private:

    /**
    * ConstructL
    * 2nd phase constructor.
    * Perform the second phase construction of a
    * CFunambolAppView object.
    * @param aRect The rectangle this view will be drawn to.
    */
    void ConstructL(const TRect& aRect, CFunambolAppUi* aAppUI);

    /**
    * CFunambolAppView.
    * C++ default constructor.
    */
    CFunambolAppView();
    
    
    void LoadListboxData();
    
    void RefreshItemText(const TInt aItemIndex, const TDesC& aText, const TBool aDimmed = EFalse);
    
    void RefreshAllListboxItems();
    
    /**
     * Returns the index of the corresponding source, in the listbox array.
     * Returns -1 (KErrNotFound) if not found.
     */
    TInt GetIndexFromSourceID(const TInt aSourceID);
    
    
    int appendSourcePane(const TInt aSourceID);
    
    int removeSourcePane(const TInt aSourceID);
    
    int countSourcesDisplayed();

    
public:

    /**
    * NewL.
    * Two-phased constructor.
    * Create a CFunambolAppView object, which will draw itself to aRect.
    * @param aRect The rectangle this view will be drawn to.
    * @return a pointer to the created instance of CFunambolAppView.
    */
    static CFunambolAppView* NewL(const TRect& aRect, CFunambolAppUi* aAppUI);

    /**
    * NewLC.
    * Two-phased constructor.
    * Create a CFunambolAppView object, which will draw itself
    * to aRect.
    * @param aRect Rectangle this view will be drawn to.
    * @return A pointer to the created instance of CFunambolAppView.
    */
    static CFunambolAppView* NewLC(const TRect& aRect, CFunambolAppUi* aAppUI);

    /**
    * ~CFunambolAppView
    * Virtual Destructor.
    */
    virtual ~CFunambolAppView();


// --------- Functions from base classes -------------

    /**
    * From CCoeControl, Draw
    * Draw this CFunambolAppView to the screen.
    * @param aRect the rectangle of this view that needs updating
    */
    void Draw( const TRect& aRect ) const;

    /**
    * From CoeControl, SizeChanged.
    * Called by framework when the view size is changed.
    */
    virtual void SizeChanged();

    TTypeUid::Ptr CFunambolAppView::MopSupplyObject(TTypeUid aId);

    /// Overrided to capture the key events: up/down arrow, selection key.
    TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);
    
    /// Returns number of controls inside this container, now = 1.
    TInt CountComponentControls() const;
    
    /// Returns the internal control given its index.
    CCoeControl* ComponentControl(TInt aIndex) const;

    
    
    /**
     * Creates the listbox as a CAknSingleLargeStyleListBox.
     * Loads data statically from resources, see Funambol.rss.
     */
    void CreateListBoxL(const TRect&/* aRect*/);
    
    void refreshScreen();
    
    bool isSourceDisplayed(const TInt aSourceID);
    
    /// Refresh the status text of the source, with the given text.
    void RefreshSourceStatus(const TInt aSourceID,  const TDesC& aText, const TBool aDimmed = EFalse);
    
    /// Refresh the status text of the source, with the buffered status text (the last one).
    void RefreshSourceBufferedStatus(const TInt aSourceID,  const TBool aDimmed = EFalse);
    
    TInt GetCurrentItemSelected();
    TInt GetCurrentSourceSelected();
};

#endif // __FUNAMBOLAPPVIEW_h__

// End of File
