/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

/*
* ============================================================================
*  Name     : CImapEmailEngine from InternetEmailAppUi.h
*  Part of  : InternetEmail
*  Created  : 25.10.2005 by Forum Nokia
*  Description:
*     Declares UI class for application.
*  Version  : 1.1
*  Copyright: Nokia Corporation 2004
* ============================================================================
*/

#ifndef IMAPEMAILENGINE_H
#define IMAPEMAILENGINE_H

// INCLUDES
#include <e32std.h>
#include <e32base.h>

#include <msvapi.h>        //of MsvSession
#include <imapset.h>

#include <msventry.h>
#include <msvids.h>
#include <mtclreg.h>    // for CClientMtmRegistry
#include <mtclbase.h>   // for CBaseMtm
#include <mtmuibas.h>   // for CBaseMtmUi
#include <mtuireg.h>    // for CMtmUiRegistry
#include <miutset.h>

#include <impcmtm.h>    // for IMAP( commands )

// FORWARD DECLARATIONS
class CFunambolAppUi;
class CSyncEngine;

// CONSTANTS

enum TInternetEmailEngineEvent
{
    ERemoteCountChanged,
};

// CLASS DECLARATION

/*
* ============================================================================
*  Name     : CImapEmailEngine from InternetEmailEngine.h
*  Part of  : InternetEmail
*  Created  : 09/11/2003 by Forum Nokia
*  Implementation notes:
*        Statemachine which handles events common for both
*        implemented protocols IMAP and POP. Note the selection
*        which protocol to use is one time only and load and store
*        functionality have been cutted off for maintaining focus
*        on basic receive-and-handle-with-mtm-generic-view functionality.
*
*  Version  : 1.0
*  Copyright: Nokia Corporation
* ============================================================================
*/
class CImapEmailEngine : public CActive, public MMsvSessionObserver, public MMsvEntryObserver
{
public: // // Constructors and destructor
    static CImapEmailEngine* NewL(CSyncEngine* syncEngine, CFunambolAppUi* appUi,
                                  TInt sourceId);
    //static CImapEmailEngine* NewLC();
    ~CImapEmailEngine();

public: // New functions
        // MAIN USER API

    /*
    * void  RemoteFetchL()
    *
    * Initiates remote mailbox fetch process
    *
    */
    void  RemoteFetchL();

    // For view handling( see InternetEmailContainer )
    TPtrC RemoteEmailTextL( const TInt aIndex);
    TPtrC RemoteEmailSenderL( const TInt aIndex);
    TInt  RemoteEmailCount();

    TBool IsEngineReady();

    /*
    * TBool CheckIfExistsL()
    *    Checks if service entry( Email settings ) exists for
    *   given internet email protocol type.
    *
    * @return TBool ETrue            if protocol of given type exists
    *
    */
    TBool CheckIfExistsL();

    void CancelOperation();
    void Queue();

    void PrepareL();

    bool CheckSyncInProgress();


public: // Functions from base classes
    // from CActive
    void RunL();
    void DoCancel();
    TInt RunError( const TInt aError);

    // from MMsvSessionObserver
    void HandleSessionEventL(TMsvSessionEvent aEvent,TAny* aArg1,TAny* aArg2,TAny* aArg3);

    // from MMsvEntryObserver
    void HandleEntryEventL(TMsvEntryEvent aEvent, TAny *aArg1, TAny *aArg2, TAny *aArg3);

// private
private:
    CImapEmailEngine(CSyncEngine* syncEngine, CFunambolAppUi* appUi, TInt sourceId);
    void ConstructL();
    TMsvId FindServiceL();
    TMsvId FindFolderThenServiceL();
    void MailCommandL( const TInt aCommand, TDes8& aParams );
    void LoadMtmL();
    CBaseMtm* InstantiateClientMtmL( TMsvId aService, const TUid aType );
    void GetMailBodyL(void);
    
    //for update
    CMsvEntrySelection* UpdateEntriesL(const TMsvId& aServiceId);
    void UpdateRemoteCountL();

    /**
     * Notify the user if new emails are available at the end of the sync.
     * The notification is generated using the notification ECOM plugin, thus 
     * is is mostly handled by the system
     *
     * Return true on success
     */
    TBool notifyUserIfNecessary();

// private types
private:

    // internal state indication
    enum TInternalState
    {
        EInitialising,
        EReadyButNeedsProtocol,
        EReady,
        EConnecting,
        EFetching,
        EGetMailBody,       // introduced to get mail body
        EGettingMailBody,   // wait for bkground request to complete 
        EDisconnecting,
        EDone,
        ECanceling
    };

private: //Data
    CMsvSession* iMsvSession; //the server session itself, our highway to kernel space 
    CClientMtmRegistry* iClientReg; //registry used to find client mtm of given protocol type
    CMtmUiRegistry* iUiReg;    //registry used to instantiante views of given mtm
    CMsvOperation* iMsvOp;    //long lasting email operation object, used e.g. for cancelling requests
    
    TMsvId iServiceId;        //service id of either protocol used e.g. as 
    TMsvId iId;                //id of the mailbox( POP ) or inbox( IMAP ) used as a root of messagelist
    TInternalState iState;     //the state of the statemachine we implement by this engine

    CMsvEntrySelection* iRemoteEntries;    //array of entries used also as mailcommmand argument
    TInt iRemoteCount;                     //count of message entries in given folder or mailbox
    
    CBaseMtm*       iMtm;            //abstract mtm for use of both pop and imap    
    CMsvEntry*      iEntry;          //current selected entry 
    RArray<TMsvId> iEmailIDs;        // Array of all email IDs received during sync.

    // next members not needed in one protocol implementation
    TBool           iHasImapInbox;       //boot have we made folder sync and got inbox?
    TUid            iProtocolUid;        //uid of selected protocol for easier abstraction

    // Number of items received during a sync
    TInt            iTotalServerItems;

    CFunambolAppUi* appUi;
    TInt            sourceId;
    TInt            iLastErrorCode;
    TBool           iSyncInProgress;
    CSyncEngine*    syncEngine;
    
};

#endif


