/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef PIM_OPTIONS_DIALOG
#define PIM_OPTIONS_DIALOG

#include <akndialog.h> 
#include <aknlists.h>
#include <aknsettingitemlist.h> 

#include "CommonOptionsSettingList.h"

// FORWARD DECLARATIONS
class CFunambolAppUi;

using namespace Funambol;


/**
 * The PIM options dialog.
 */
class CPIMOptionsDialog : public CAknDialog
{

private:
    
    /// Pointer to the object that created this instance, the appUI.
    /// Used to callback methods to the appUI, which is the controller.
    CFunambolAppUi*             iAppUI;
    
    CCommonOptionsSettingList*  iSettingList;
    
    /// Defines the source ID corresponding to this dialog: Contacts, Calendar or Notes.
    TInt                        iSourceID;
    
    /// Useful to store also the name, once for all.
    StringBuffer                iSourceName;
    
    TBool                       iSyncEnabled;
    TSyncMode                   iSyncMode;
    
    
    // Default constructor/destructor
    CPIMOptionsDialog();
    ~CPIMOptionsDialog();
    
    /// Calls the CAknDialog::ConstructL, with the menu resource.
    void ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI, const TInt aSourceID);
    
    
    /// Overrided to create the setting list at runtime. 
    void PreLayoutDynInitL();
    
    // Overrided to handle dialog's menu commands.
    // Not used: we don't use a menu!
    //void ProcessCommandL(TInt aCommand);
    
    /// Invoked before leaving the dialog. Decides if we need to save or cancel.
    TBool OkToExitL(TInt aButtonId);
    
    /// Overrided to capture the key events: up/down arrow, selection key.
    TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);
    
    
    /**
     * Loads data from client configuration into the UI dialog.
     */
    void  load();
    
    /**
     * Saves data from UI dialog into the client configuration.
     * If data is unchanged, exits with EFalse.
     * @return  ETrue if settings saved correctly, EFalse if not.
     */
    TBool save();
    
    
public:
    
    /**
     * Creates a new CPIMOptionsDialog object.
     * @param aMenuTitleResourceId  the resource ID of the menubar for this dialog
     * @param aAppUI                pointer to the FunambolAppUI object that called this method
     */
    static CPIMOptionsDialog* NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI, const TInt aSourceID);
    
};

#endif  // PIM_OPTIONS_DIALOG
