#ifndef PHOTOOPTIONSDIALOG_H_
#define PHOTOOPTIONSDIALOG_H_

#include <akndialog.h> 
#include <aknlists.h>
#include <aknsettingitemlist.h> 

#include "CommonOptionsSettingList.h"

// FORWARD DECLARATIONS
class CFunambolAppUi;

using namespace Funambol;


/**
 * The email options dialog.
 */
class CPhotoOptionsDialog : public CAknDialog
{

private:
    
    /// Pointer to the object that created this instance, the appUI.
    /// Used to callback methods to the appUI, which is the controller.
    CFunambolAppUi*             iAppUI;
    
    CCommonOptionsSettingList*  iSettingList;
    
    TBool                       iSyncEnabled;
    TSyncMode                   iSyncMode;
    TInt                        iMemoryInUse;
    TBool                       iMassStorageAvailable;
    TBool                       iIncludeOlderPictures;

    // Default constructor/destructor
    CPhotoOptionsDialog();
    ~CPhotoOptionsDialog();
    
    /// Calls the CAknDialog::ConstructL, with the menu resource.
    void ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI);
    
    
    /// Overrided to create the setting list at runtime. 
    void PreLayoutDynInitL();
    
    // Overrided to handle dialog's menu commands.
    // Not used: we don't use a menu!
    //void ProcessCommandL(TInt aCommand);
    
    /// Invoked before leaving the dialog. Decides if we need to save or cancel.
    TBool OkToExitL(TInt aButtonId);
    
    /// Overrided to capture the key events: up/down arrow, selection key.
    TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);
    
    /**
     * Execute operations to edit the current item.
     * Pictures sync direction is fixed, so it won't open up.
     */
    void EditCurrentItem();
    
    /**
     * Loads data from client configuration into the UI dialog.
     */
    void  load();
    
    /**
     * Saves data from UI dialog into the client configuration.
     * If data is unchanged, exits with EFalse.
     * @return  ETrue if settings saved correctly, EFalse if not.
     */
    TBool save();
    
public:
    
    /**
     * Creates a new CPhotoOptionsDialog object.
     * @param aMenuTitleResourceId  the resource ID of the menubar for this dialog
     * @param aAppUI                pointer to the FunambolAppUI object that called this method
     */
    static CPhotoOptionsDialog* NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI);  
};

#endif /*PHOTOOPTIONSDIALOG_H_*/
