/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef PICTURESSOURCESYNCMLDATA_H_
#define PICTURESSOURCESYNCMLDATA_H_


#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "base/util/ItemContainer.h"
#include "spds/FileData.h"
#include "client/CacheSyncSource.h"
#include "client/MediaSyncSourceSyncMLData.h"

BEGIN_NAMESPACE

/**
 * This class extends the MediaSyncSourceSyncMLData, implementing the sync for
 * picture files. The source behavior is defined in the parent class, 
 * the only method implemented is the one for the filtering of item types.
 * 
 * NOTE: this class is manteined only for backward compatibility, as well as
 * the MediaSyncSourceSyncMLData class. It will be removed in future.
 */
class PicturesSyncSourceSyncMLData : public MediaSyncSourceSyncMLData
{
public:
    PicturesSyncSourceSyncMLData(const WCHAR* name, 
                       AbstractSyncSourceConfig* sc, 
                       const StringBuffer& aDir);
    
    ~PicturesSyncSourceSyncMLData() {};

    /**
     * Overrides MediaSyncSourceSyncMLData::beginSync().
     * Checks if the new "media-http-upload" Server capability is found,
     * in this case current sync is aborted (a new pictures sync will be 
     * started.
     */
    int beginSync();
    
    
protected:
    
    /**
     * Overrides MediaSyncSource::filterOutgoingItem().
     * Filter out Symbian thumbnail folders, and files that are not images.
     * The only files accepted are: "jpg, jpeg, gif, png".
     * The filtering is done on files extension (case insensitive).
     * NOTE: this is only STATIC filtering.
     *       Meaning that these filters CANNOT be changed/disabled anytime by the Client,
     *       otherwise a change in the filter may result in the Client to send 
     *       deleted items for files fitered out.
     *       See dynamicFilterItem() for dynamic filtering.
     * 
     * @param fullName  the full path + name of the file to check
     * @param st        reference to struct stat for current file
     * @return          true if the item has to be filtered out (skipped)
     *                  false if the item is ok
     */
    bool filterOutgoingItem(const StringBuffer& fullName, struct stat& st);
    
};

END_NAMESPACE

#endif /*PICTURESSOURCESYNCMLDATA_H_*/
