/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef _SMSPUSHENGINE_H_
#define _SMSPUSHENGINE_H_

#include <e32std.h>
#include <e32base.h>
#include <e32cons.h>
#include <es_sock.h>
#include <f32file.h>

class CPushEngine;
class CSMSPushSubscriber;

using namespace Funambol;

/**
 * Provides functionality for SMS push.
 * Calls iPushEngine->onSmsNotificationReceived() with the list of source ids
 * when a SMS push notification is got via subscriber 
 */
class CSMSPushEngine : public CBase
{
    
private:
    
    CPushEngine*            iPushEngine;
    CSMSPushSubscriber*     iPushSubscriber; 

    // timer for push server watchdog
    CPeriodic* iTimer; 

    // True if is currently watching for SMS push.
    bool iRunning;

    /**
     * Perform the second phase construction.
     */
    void ConstructL();

    /**
     * Perform the first phase of two phase construction.
     */
    CSMSPushEngine(CPushEngine* aPushEngine);
    
   
    /**
     * Checks for push server process running (wrapper to utils CheckProcessRunning)
     */
    TBool CheckPushServerProcess(); 
 
    /**
     * Starts push server instance
     */
    void StartPushServerProcessL(); 

    /**
     *
     */
    static TInt PushServerWatchDog(TAny *aPtr); 

public:
    
    /**
     * Create a CSMSPushEngine object.
     */
    static CSMSPushEngine* NewL(CPushEngine* aPushEngine);

    /**
     * Create a CSMSPushEngine object.
     */
    static CSMSPushEngine* NewLC(CPushEngine* aPushEngine);

    /**
     * Destroy the object
     */
    ~CSMSPushEngine();
    
    
    /// Starts SMS push subscriber.
    void start();
    
    /// Stops SMS push subscriber.
    void stop(TBool aWatchDogStopFlag=EFalse);
    
    /// Returns true if SMS push subscribe is active (we're listening for notifications).
    bool isRunning() { return iRunning; }

    // notifies sync client push engine
    void onSMSPushReceived(RArray<TInt>& aSourceList);
};


#endif /*_SMSPUSHENGINE_H_*/
