/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef SYNC_ACTIVE_OBJECT_INC
#define SYNC_ACTIVE_OBJECT_INC

#include <eikenv.h>
#include "base/util/StringBuffer.h"

#include "commonDefs.h"
#include "customization.h"

// Forward class declaration
class CSyncEngine;
class TimeoutTimer;

/**
 * This class allows executing a sync in a separate ActiveObject with a timeout.
 * It is meant to be used only by the SyncEngine.
 */

class SyncActiveObject : public CActive
{

private:

	enum State {
        EDone,
        ECancelled,
        ESync,
        EStopped,
        ENone
    };
    
    static TInt     iSourceId; 
    StringBuffer    iCacheFile;
    TInt            status;
    RTimer          iTimer;
    static int      itemsSuccesfullySynced;
    
    static State             iState;
    static CSyncEngine*      iSyncEngine;
    static SyncActiveObject* syncAO;
    static TimeoutTimer*     tt;
    
public:
    static void syncLD(CSyncEngine* aSyncEngine, const TInt aSourceId, const StringBuffer& aCacheFile = NULL);
    static bool CheckSyncInProgress();
    
    /**
     * Call this method to abort the current sync.
     * 
     * We need to stop this process, so we set the lastErroMessage to 'KErrCancel' 
     * and the sync process will periodically check it. 
     * In case it's equal to KErrCancel the sync process will leave (User::Leave(KErrCancel))
     */
    static void CancelSync();
    
private:
    static SyncActiveObject* NewL(const TInt aSourceId, const StringBuffer& cacheFile);
    static SyncActiveObject* NewLC(const TInt aSourceId, const StringBuffer& cacheFile);
    ~SyncActiveObject();

    SyncActiveObject(const TInt aSourceId, const StringBuffer& cacheFile);
    void ConstructL();
    void syncL();
    void RunL();
    void sync();
    void Stop();
    
    /**
     * Always call this private method when a sync ends.
     * Deletes the active object and calls SyncEngine::syncEnded() to refresh Client's UI.
     * @param aborted  true if we stopped the sync in progress (timeout or Cancel sync)
     * @param status   the return value of the sync
     */
    static void syncEnded(const bool aborted, const TInt status);

protected:
    void DoCancel();

    friend class TimeoutTimer;
};

#endif

