/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef _XMLRPC_ENGINE_H_
#define _XMLRPC_ENGINE_H_

#include <e32base.h>
#include <e32def.h>
#include <e32std.h>
#include <in_sock.h>
#include <apselect.h>
#include <apgtask.h>
#include <coemain.h>
#include <apgcli.h>  

USE_NAMESPACE

class CFunambolAppUi;

typedef StringBuffer (*xmlRpcCallback) (void);

struct xmlRpcHandler
{
    char* handlerName;
    xmlRpcCallback callBack;
};
  
class CXmlRpcEngine : public CActive
{

public:

    static CXmlRpcEngine* NewL(CFunambolAppUi* aAppUi);
    ~CXmlRpcEngine();
    void StartL();
    void Stop();

    // RPC callback handlers
    static StringBuffer GetUserAccountCallBack();
    
private:

    static CXmlRpcEngine* NewLC(CFunambolAppUi* aAppUi);
    void ConstructL();
    CXmlRpcEngine(CFunambolAppUi* aAppUi);

    void RunL();
    void DoCancel(); 

    void MakeAccept();
    void MakeBind();

    bool ParseXmlMsg();

    
private:
  
    enum TXmlRpcEngineStatus
    {
        EXmlRcpStopped,
        EXmlRpcMessageWait,
        EXmlRpcMessageParse,
        EXmlRpcSendResult
    };
    
    static const TInt KSyncClientPort = 50005;
    static const TInt KSocketBufSize = 4096;
    static const TInt KListenBackLog = 1; 
    // callback registration table
    static xmlRpcHandler xmlRpcHandlersTable [];
    // XML-RPC static error messages
    static const char* xmlRpcInvalidMessageResponse;
    static const char* xmlRpcInvalidMethodResponse;
    // XML-RPC message formatting templates
    static const char* xmlRpcResponseFmt;
    static const char* xmlRcpResponseStringParamFmt;
    static const char* xmlRcpResponseAccountStructParamFmt;
    // content length header template
    static const char* contentLengthHdrFmt;
    

    RConnection iConnection;
    RSocketServ iSocketSession;
    RSocket     iListen, iSocket;
    TBuf8<KSocketBufSize>   iSocketBuf;
    TSockXfrLength          iDummy;
    TInetAddr               iAddress;
    CPeriodic*              iBindTimer;
    StringBuffer            iSocketData;
  
    TXmlRpcEngineStatus iState;
    CFunambolAppUi* iAppUi; 

    TBool iParserError;
    TBool iMessageParseComplete;

    StringBuffer iXmlRpcMsg;        // incoming XML-RPC message
    StringBuffer iXmlRpcResponse;   // XML-RPC response to client
    xmlRpcCallback iXmlRcpCallback; // RPC callback
};

#endif
