/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef INCL_CUSTOMIZATION_DEFS
#define INCL_CUSTOMIZATION_DEFS


//
// --------------- Definition to customize client ----------------
//

// For all strings localization
// Add here new language files to be included
#if defined LANGUAGE_EN | defined LANGUAGE_SC
    #include "strings_en.rls"
#elif defined LANGUAGE_DE
    #include "strings_de.rls"
#endif


#define APPLICATION_NAME                  "Funambol Symbian Sync Client"    /**< Application name: used for UserAgent, About dialog, Log title */
#define WSYNC_PROFILE_NAME                TEXT("funambol")                  /**< SyncML profile and IMAP Account name */
#define SYNC_PROFILE_NAME                 "funambol"                        /**< SyncML profile and IMAP Account name */
#define MANUFACTURER_NAME                 "Funambol"                        /**< Manufacturer name, used by DevInfo */

#define SYNC_URI                          "http://my.funambol.com/sync"
#define CARD_REMOTE_URI                   "card"
#define CAL_REMOTE_URI                    "cal"
#define NOTES_REMOTE_URI                  "note"
#define PICTURES_REMOTE_URI               "picture"
#define USE_PICTURES_RAW_FILES            0                                 /**< set to '1' to send pictures as raw files(no file-object) */
#define DEFAULT_PICTURES_SYNC_MODE        SYNC_SMART_ONE_WAY_FROM_CLIENT_TEXT
#define CTP_PORT                          4745

#define ENABLE_CREDENTIALS_VALIDITY_CHECK 1    /**< '1' to locally check credentials validity */
#define ENABLE_AT_VALIDITY_CHECK          0    /**< '1' to check if username contains @ */
#define ENABLE_SERVER_URL                 1    /**< '1' to show 'Server URL' in account dialog */
#define ENABLE_SOURCE_MAIL                0    /**< '1' to view the Emails panel by default. Used only by ClientSettings */
#define ENABLE_SOURCE_NOTES               1    /**< '1' to view the Notes panel by default. Used only by ClientSettings */
#define ENABLE_SOURCE_PHOTOS        	  0    /**< '1' to view the Pictures panel by default. Used only by ClientSettings (also see DYNAMICALLY_SHOW_PICTURES) */
#define ENABLE_SOURCE_CONFIG_IN_SYNCALL   0    /**< '1' to allow sync of config when clicking SyncAll - disabled now, config source must be blocking! */
#define ENABLE_LOG_SETTINGS               1    /**< '1' to view the 'Logging' selection in Settings dialog */
#define ENABLE_HELP                       0    /**< '1' to show 'Help' menu button */
#define ENABLE_SYNC_ALL                   0    /**< '1' to show the sync all button */
#define ENABLE_JAVA_MAIL_CLIENT           0    /**< '1' to allow java email client support */
#define DEFAULT_SYNC_USING                0    /**< 0 = PUSH, 1 = MANUAL, 2 = SCHEDULED */
#define DEFAULT_PUSH_TYPE                 7    /**< 0 = none, 1 = CTP only, 2 = SMS only, 4 = C2S only, 7 = all (bit-mask) */

#define DEFAULT_LOG_LEVEL                 1    /**< 0 = ERROR, 1 = INFO, 2 = DEBUG */
#define LOG_FILE_PATH                     "C:\\data\\"
#define LOG_FILE_NAME                     "funambol-log.txt"

// About dialog:
#define ENABLE_ABOUT_MAIN_WEB_SITE        1    /**< '1' to view the web site address in about dialog */
#define ENABLE_ABOUT_COPYRIGHT            1    /**< '1' to view the Copyright license text in about dialog */
#define ENABLE_ABOUT_POWERED_BY           0    /**< '1' to view the 'Powered by' text in about dialog */
#define ENABLE_ABOUT_POWERED_BY_WEB_SITE  0    /**< '1' to view the web site address in about dialog */
#define ENABLE_ABOUT_LICENSE              1    /**< '1' to view the license text in about dialog */
#define PRE_ALPHA_RELEASE                 0    /**< Displays "(pre-alpha)" in the about screen */
#define ALPHA_RELEASE                     0    /**< Displays "(alpha)" in the about screen */
#define BETA_RELEASE                      0    /**< Displays "(beta)" in the about screen */

#define USE_MAIN_SCREEN_BIG_ICONS         1    /**< '1' to use big images on the main screen */
#define SYNC_PROCESS_TIMEOUT              0    /** Timeout (in seconds) for the entire sync process. 0 means "no timeout". */
#define CREDENTIALS_CHECK_TIMEOUT         120  /** Timeout (in seconds) for credentials check */
#define CP_INTERVAL_SECONDS               60   /** Interval (in seconds) for the Client Push */

// CTP connection parameters
#define CTP_READY_INTERVAL                600  /** Interval (in seconds) for the CTP [ready] msg */
#define CTP_CONN_TIMEOUT                  0
#define CTP_CMD_TIMEOUT                   60
#define CTP_NOTIFY_TIMEOUT                120
#define CTP_RETRY_INTERVAL                60
#define CTP_MAX_RETRY                     1200

#define DEFAULT_SCHEDULER_INTERVAL        30   /** Default interval (in minutes) for the scheduled sync */
#define MAX_SYNCML_MSG_SIZE               65536 /** maximum size in bytes of a single SyncML message */

// pictures sync
#define MAX_IMAGE_SIZE                    50000   /** maximum size in KBytes of a single photo item: 50MB (so it's like we don't have a limit) */
#define DYNAMICALLY_SHOW_PICTURES         1       /**< if 1, will automatically show/hide the pictures panel, at the end of sync */
#define ENABLE_PICTURES_CLEANUP           0       /** if 1, will show the cleanup of pictures in cleanup dialog (0 is default) */
#define DEFAULT_INCLUDE_OLDER_PICTURES    0       /** if 0, will filter out the pictures before installation time - can be changed via settings */

// MBM related defines (bmp images).
// They depend on the mmp file name, which MUST be changed to make the application indipendent.
#include <FunambolSync.mbg>               // The header file of bitmaps, required for enumerations
#define BITMAPS_MBM_PATH                  "\\resource\\apps\\FunambolSync.mbm"
#define MAIN_LOGO_BMP_ID                  EMbmFunambolsyncMainlogo
#define MAIN_LOGO_BMP_MASK_ID             EMbmFunambolsyncMainlogo_mask

// UID3 and MIF icons related defines.
// They depend on the UID3, which MUST be changed to make the application indipendent.
#include <Funambol_0x2001D2C7.mbg>        // Includes mif enumerations.
#define LOCALISABLE_RESOURCE_FILE         "\\resource\\apps\\Funambol_0x2001D2C7"
#define LOCALISABLE_RESOURCE_FILE_MIF     "\\resource\\apps\\Funambol_0x2001D2C7.mif"
#define MAIN_LOGO_ICON_ID                 EMbmFunambol_0x2001d2c7Mainlogo
#define MAIL_ICON_ID                      EMbmFunambol_0x2001d2c7Icon_mail
#define MAIL_DIMMED_ICON_ID               EMbmFunambol_0x2001d2c7Icon_mail_dimmed
#define MAIL_SETTINGS_ICON_ID             EMbmFunambol_0x2001d2c7Icon_mail_settings
#define CONTACTS_ICON_ID                  EMbmFunambol_0x2001d2c7Icon_contacts
#define CONTACTS_DIMMED_ICON_ID           EMbmFunambol_0x2001d2c7Icon_contacts_dimmed
#define CONTACTS_SETTINGS_ICON_ID         EMbmFunambol_0x2001d2c7Icon_contacts_settings
#define CALENDAR_ICON_ID                  EMbmFunambol_0x2001d2c7Icon_calendartasks
#define CALENDAR_DIMMED_ICON_ID           EMbmFunambol_0x2001d2c7Icon_calendartasks_dimmed
#define CALENDAR_SETTINGS_ICON_ID         EMbmFunambol_0x2001d2c7Icon_calendartasks_settings
#define NOTES_ICON_ID                     EMbmFunambol_0x2001d2c7Icon_notes
#define NOTES_DIMMED_ICON_ID              EMbmFunambol_0x2001d2c7Icon_notes_dimmed
#define NOTES_SETTINGS_ICON_ID            EMbmFunambol_0x2001d2c7Icon_notes_settings
#define PHOTOS_ICON_ID                    EMbmFunambol_0x2001d2c7Icon_photos
#define PHOTOS_DIMMED_ICON_ID             EMbmFunambol_0x2001d2c7Icon_photos_dimmed
#define PHOTOS_SETTINGS_ICON_ID           EMbmFunambol_0x2001d2c7Icon_photos_settings
#define SYNC_METHOD_ICON_ID               EMbmFunambol_0x2001d2c7Icon_sync
#define LOGGING_ICON_ID                   EMbmFunambol_0x2001d2c7Icon_logs


// IMAP EMail related defs:
#if ENABLE_SOURCE_MAIL
#define IMAP_SERVER                       "imap.gmail.com"
#define IMAP_PORT                         993
#define IMAP_SSL                          1
#define IMAP_TLS                          0
#define SMTP_SERVER                       "smtp.gmail.com"
#define SMTP_PORT                         465
#define SMTP_SSL                          1
#define SMTP_TLS                          0
#define IMAP_RETRIEVE_BODY                1               /**< '1' to retrieve mail body, not only headers */
#define MAX_EMAIL_SIZE                    0x00010000      /**< 64k = max email body downloaded upfront (was 0x7FFFFFFF) */
#endif

// jad path on device
#define EMAIL_CLIENT_JAD_PATH             "C:\\Data\\Installs\\FunambolEmailClient.jad"
// midlet name
#define MIDLET_NAME                       TEXT("Funambol")
// port where send data to start jam
#define MIDLET_PUSH_PORT                  50001

// updater component
#define UPDATE_SOFTWARE_URL_RESOURCE      "/sapi/profile/client?action=get-update-info"
#ifdef __S60_30__
#define UPDATER_COMPONENT                 "sym"
#else
#define UPDATER_COMPONENT                 "sym_fp2"
#endif

#define SMS_PUSH_PORT                     50011         /**< port used for SMS push */
#endif /*INCL_CUSTOMIZATION_DEFS*/
