/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef _SMSPUSHLISTENER_H_
#define _SMSPUSHLISTENER_H_

#include <e32std.h>
#include <e32base.h>
#include <e32property.h>        // P&S API
#include <es_sock.h>
#include <f32file.h>

#include <msvapi.h>         // of MsvSession
#include <mtclreg.h>        // CClientMtmRegistry

#include "base/SymbianLog.h"
#include "base/symbianadapter.h"
#include "base/util/symbianUtils.h"
#include "base/util/stringUtils.h"
#include "utils.h"
#include "commonDefs.h"         // source IDs
#include "customization.h"

#include "SMSPushProperty.h"

// log file 
#define PUSH_SERVER_LOG_PATH "C:\\Data\\"
#define PUSH_SERVER_LOG_NAME "push_server.log"
#define PUSH_SERVER_APP_NAME "PushServer"

class RFs;

using namespace Funambol;

/**
 * Provides functionality for SMS push.
 */
class CSMSPushListener : public CActive, public MMsvSessionObserver
{
    
private:

    // True if is currently watching for SMS push.
    bool iRunning;

    RSocketServ iSocketServer;
    RSocket iSmsSocket;
    RFs iFs;
    TPckgBuf<TUint> iSbuf;

    enum TSMSState
    {
        EListenerCreated,
        EWaitingForSMS,
        EAcknowledgingSMS,
        EListenerStopped
    };

    TSMSState iState;
    TBool iStartSyncOnPush; 

    CMsvSession* iMsvSession;   //the server session
    CClientMtmRegistry* iClientMtmReg;
   
    // published property
    RProperty iPushProp;
    
    /**
     * Perform the second phase construction.
     */
    void ConstructL();

    /**
     * Perform the first phase of two phase construction.
     */
    CSMSPushListener();
    
    /**
     * Called when a SMS push notification is received.
     */
    void onPushReceived(const char* body, TInt length);
    
    /**
     * Create connection with socket server and file server, opens SMS socket
     */
    void SetupConnectionL();

    /**
     * Listen for SMS message on SMS_PUSH_PORT
     */
    void ListenSmsPushL();
    
    /**
     * Parses the WSP (Wireless Session Protocol) headers before the SMS push pkg0 message.
     * Expects a buffer of bytes (msg) of length 'msgLength' as input.
     * - The second byte is PDU type and MUST be 0x06 (Push)
     * - The last byte is the content type, and MUST be 0x85 (0x80 | 0x05 = x-wap-application:push.syncml)
     *   (see http://www.wapforum.org/wina/push-app-id.htm)
     * 
     * For example: [0x01 0x06 0x03 0xce 0xaf 0x85 <body>]
     *    0x01 = Push ID
     *    0x06 = PDU Type (Push)
     *    0x03 = Headers length (3 bytes)
     * 
     * @param msg       pointer to the first byte of the msg to parse.
     * #param msgLength the msg length.
     * @return          the length (a value > 0) of the WSP headers parsed, if the msg is a valid SMS push message for SyncML Push
     *                  a value < 0 if the msg passed is NOT a SMS push message for SyncML Push (or in case of errors)
     */
    int parseHeadersWSP(const char* msg, const int msgLength);

    // CActive functions
    void DoCancel();
    void RunL();
    TInt RunError(TInt aError);
    
    
    /**
     * Scans the inbox folder and removes any push SMS found.
     * SMS are recognized as push message for us checking the message WSP headers.
     * Each push message is parsed and a sync will be triggered for the required sources.
     * After that, the SMS push message is removed.
     */
    void cleanSMSFromInboxL();
    
    /**
     * Opens the SMS message and returns its body.
     * The message body can be binary, so it's returned as a new allocated char* 
     * together with its length (bodyLength).
     * @param aEntry      the entry in the Message Server index, corresponding to the SMS to read
     * @param bodyLength  [OUT] the length of body read (in byte)
     * @return            the message body (new allocated char*)
     */
    char* getSMSBodyL(const TMsvEntry& aEntry, int* bodyLength);

    
public:
    
    /**
     * Create a CSMSPushListener object.
     */
    static CSMSPushListener* NewL();

    /**
     * Create a CSMSPushListener object.
     */
    static CSMSPushListener* NewLC();

    /**
     * Destroy the object
     */
    ~CSMSPushListener();
    
    /// Starts watching for SMS push messages.
    void StartL();
    
    /// Stops watching for SMS push messages.
    void Stop(TBool aStopFlag=EFalse);
    
    /// Returns true if SMS push is active (we're listening for notifications).
    bool isRunning() { return iRunning; }
    
    // from MMsvSessionObserver
    void HandleSessionEventL(TMsvSessionEvent aEvent,TAny* aArg1,TAny* aArg2,TAny* aArg3);
};

#endif /*_SMSPUSHLISTENER_H_*/
