/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


// sms 3rd party API headers
#include <smsuaddr.h>
#include <gsmubuf.h>
#include <gsmumsg.h>
#include <smsustrm.h>

#include <smsclnt.h>            // class CSmsClientMtm
#include <smut.h>               // defines KUidMsgTypeSMS
#include <smuthdr.h>            // CSmsHeader
#include <cmsvrecipientlist.h>  // CMsvRecipientList

#include <stdio.h>              // sprintf
#include <txtrich.h>            // RichText

#include "base/SymbianLog.h"
#include "base/symbianadapter.h"
#include "base/util/symbianUtils.h"
#include "base/util/stringUtils.h"
#include "push/CTPService.h"
#include "utils.h"
#include "commonDefs.h"         // source IDs
#include "customization.h"

#include "SMSPushListener.h"

static void hexDump(char *buf, int len);


CSMSPushListener* CSMSPushListener::NewL()
{
    CSMSPushListener* self = CSMSPushListener::NewLC();
    CleanupStack::Pop( self );
    return self;
}

CSMSPushListener* CSMSPushListener::NewLC()
{
    CSMSPushListener* self = new (ELeave) CSMSPushListener();
    CleanupStack::PushL( self );
    self->ConstructL();
    return self;
}

CSMSPushListener::CSMSPushListener() : CActive(EPriorityStandard), iRunning(false), 
    iState(EListenerCreated), iStartSyncOnPush(ETrue) {}

CSMSPushListener::~CSMSPushListener() 
{
    // remove published properties
    iPushProp.Close();
    RProperty::Delete(KSMSPushPropUid, EPushReceived);

    if (iState != EListenerStopped) {    
        iFs.Close();
        iSmsSocket.Close();
        iSocketServer.Close();
        Cancel();
    }
    
    delete iClientMtmReg;
    iClientMtmReg = NULL;
    
    delete iMsvSession; //Session must be deleted last
    iMsvSession = NULL;
}

void CSMSPushListener::SetupConnectionL()
{
    LOG.debug("setting up connection");
    User::LeaveIfError(iFs.Connect());
    // Connect to sockets server: SMS messages are intercepted via sockets
    User::LeaveIfError(iSocketServer.Connect());
    // SMS socket
    User::LeaveIfError(iSmsSocket.Open(iSocketServer, KSMSAddrFamily, KSockDatagram, KSMSDatagramProtocol));
    
    // set SMS port on which we will start listening
    TSmsAddr smsAddress;
    smsAddress.SetSmsAddrFamily(ESmsAddrApplication16BitPort);
    smsAddress.SetPort(SMS_PUSH_PORT);
    TInt err = KErrNone;
 
    if ((err = iSmsSocket.Bind(smsAddress)) != KErrNone) {
        LOG.error("error binding on port %d [errno %d]" , SMS_PUSH_PORT, err);
    }
}

void CSMSPushListener::ConstructL()
{
    // create and open a session towards messaging application
    iMsvSession = CMsvSession::OpenSyncL(*this);
    iClientMtmReg = CClientMtmRegistry::NewL(*iMsvSession);
    
    CActiveScheduler::Add(this);

    // define and publish property
    LOG.debug("publishing push property");
    TInt ret = RProperty::Define(KSMSPushPropUid, EPushReceived, RProperty::EInt);
    if (ret != KErrAlreadyExists) {      
        User::LeaveIfError(ret);
    } 
 
    iPushProp.Attach(KSMSPushPropUid, EPushReceived, EOwnerThread);
    iPushProp.Set(ENullPush);
 
    SetupConnectionL();
    
    // Clean up the inbox folder
    LOG.info("Clean up inbox folder from SMS pushed");
    TRAPD(err, cleanSMSFromInboxL());
    if (err) {
        LOG.error("Error cleaning up SMS from inbox folder: code %d", err);
    }
}

void CSMSPushListener::ListenSmsPushL()
{
    LOG.debug("start listening for incoming SMS");
    // RunL will be called when a message is received
    iSbuf()=KSockSelectRead;
    iStatus = KRequestPending;
    iSmsSocket.Ioctl(KIOctlSelect, iStatus, &iSbuf, KSOLSocket);
    iState = EWaitingForSMS;
    SetActive();
}

void CSMSPushListener::StartL()
{
    switch (iState) {
        case EListenerStopped:
            // setup connection if object was stopped
            SetupConnectionL();

            break;
        case EWaitingForSMS:
        case EAcknowledgingSMS:
            if (iStartSyncOnPush) {
                LOG.error("SMSListener: start command received while already active");

                return;
            }

            break;
        default:
            break;
    }

    // if push service was disbled reenable it on start call
    if (iStartSyncOnPush == EFalse) {
        //LOG.debug("enabling sync start on SMS push");
        iStartSyncOnPush = ETrue;
    } 

    // start listening
    if (!IsActive()) {
        ListenSmsPushL();
    }

    iRunning = true;
}

void CSMSPushListener::Stop(TBool aStopFlag)
{
    LOG.info("stopping SMS push listener");
 
    if (aStopFlag) {    
        //LOG.debug("Stopping SMS socket server");
        iFs.Close();
        iSmsSocket.Close();
        iSocketServer.Close();

        Cancel();

        iState = EListenerStopped;
    } else {
        // continue receiving SMS but disable sync start
        //LOG.debug("sync start on SMS push disabled");
        iStartSyncOnPush = EFalse;
    }
    
    iRunning = false;
}

TInt CSMSPushListener::RunError(TInt aError) 
{
    LOG.error("%s: RunL() error: %d", PUSH_SERVER_APP_NAME, aError);

    return KErrNone;
}

void CSMSPushListener::DoCancel()
{

}

// RunL() handles only EWaitingForSMS and EAcknowledgingSMS states
void CSMSPushListener::RunL()
{
    if (iState == EAcknowledgingSMS)
    {
        // The acknowledgement has been sent: start waiting for another SMS push 
        ListenSmsPushL();
        
    } else if (iState == EWaitingForSMS) {
        LOG.debug("%s: received SMS on port %d", PUSH_SERVER_APP_NAME, SMS_PUSH_PORT);
        CSmsBuffer* smsBuffer = CSmsBuffer::NewL();
        CleanupStack::PushL(smsBuffer);
        CSmsMessage* smsMessage = CSmsMessage::NewL(iFs, CSmsPDU::ESmsDeliver, smsBuffer);
        // smsMessage has taken ownership of smsBuffer so remove it from the cleanup stack.
        CleanupStack::Pop(smsBuffer);
        CleanupStack::PushL(smsMessage);
        
        // open socket read stream
        //LOG.debug("reading sms message contents");
        RSmsSocketReadStream readStream(iSmsSocket);
        CleanupClosePushL(readStream);
        smsMessage->InternalizeL(readStream);
        CleanupStack::PopAndDestroy(&readStream);

        // Extract the full SMS message (including headers)
        HBufC* message = HBufC::NewLC(smsMessage->Buffer().Length());
        TPtr messagePtr(message->Des());
        smsMessage->Buffer().Extract(messagePtr, 0, smsMessage->Buffer().Length());

        // Convert to multibyte
        HBufC8* message8 = HBufC8::NewLC(message->Length());
        message8->Des().Copy(message->Des());
        //LOG.debug("full sms push msg: len = %d", message8->Length());
        hexDump((char*)message8->Ptr(), message8->Length());
        
        int bodyLength = message8->Length();
        char* body = (char*)message8->Ptr();
        
        // Parse the SMS WSP headers and skip them.
        int headersLength = parseHeadersWSP(body, bodyLength);
        if (headersLength > 0) {
            //LOG.debug("WSP headers length = %d", headersLength);
            body       += headersLength;
            bodyLength -= headersLength;
        }

        // if SMS push service is enabled start sync
        // otherwise just send ack
        if (iStartSyncOnPush) {
            // --- Parse the sync notification pkg#0 and start the sync ---
            onPushReceived(body, bodyLength);
        }

        CleanupStack::PopAndDestroy(3); // message, smsMessage, message8
        readStream.Close();

        LOG.debug("%s: acknowledging receipt of the SMS message", PUSH_SERVER_APP_NAME);
        // Acknowledge receipt of the SMS message, otherwise it will be resent
        TPckgBuf<TUint> sbuf;
        iStatus = KRequestPending;
        iSmsSocket.Ioctl(KIoctlReadMessageSucceeded, iStatus, &sbuf, KSolSmsProv);
        iState = EAcknowledgingSMS;
        //LOG.debug("SMS acknowledging done");
        SetActive();
    }
}


int CSMSPushListener::parseHeadersWSP(const char* msg, const int msgLength)
{
    LOG.debug("parsing SMS message");
    if (msgLength <= 3) {
        // We expect at least 3 bytes
        //LOG.debug("Error parsing SMS WSP headers: message length is too small (%d)", msgLength);
        return -1;
    }
    
    // byte #1 is Push ID (skip)
    char* p = (char*)msg;
    p++;
    
    // byte #2 is PDU Type
    int pduType = *p;
    if (pduType != 0x06) {
        ////LOG.debug("WSP PDU Type is not 0x06 = Push");
        return -2;
    }
    p++;
    
    // byte #3 is headers length
    int headersLength = *p;
    if (headersLength >= msgLength - 3) {
        //LOG.debug("Error parsing SMS WSP headers: headers length is too big (0x%02x)", headersLength);
        return -3;
    }
    
    // Last byte is content type: must be 0x85 (0x80 | 0x05)
    // 0x80 is fixed
    // 0x05 = x-wap-application:push.syncml (see http://www.wapforum.org/wina/push-app-id.htm)
    p += headersLength;
    int contentType = *p;
    if (contentType != 0x85) {
        //LOG.debug("WSP content type is not 0x85 = x-wap-application:push.syncml");
        return -4;
    }
    
    // If here, the message is a valid SMS push message for SyncML Push
    // The total headers length
    return (headersLength + 3);
}


void CSMSPushListener::onPushReceived(const char* body, TInt length)
{
    TInt pushVal = ENullPush; 
    SyncNotification* san = new SyncNotification();

    SN_Errors ret = san->parse(body, length);
    //LOG.debug("push message received:");
    //hexDump((char*)body, length);
    if (ret != SNErr_Ok) {
        //LOG.error("Error parsing SMS notification message: SN error = %d", ret);
    }

    CTPService* pushService = CTPService::getInstance();
    ArrayList uriList = pushService->getUriListFromSAN(san);   
 
    delete san;

    // get current push flags
    iPushProp.Get( pushVal );
    TBool valueUpdate = EFalse;

    // Convert each ServerURI string into a source ID.
    for (int i=0; i < uriList.size(); i++) {
        StringBuffer* uri = (StringBuffer*)uriList[i];
        if ( !uri || !(uri->length()) ) {
            continue;
        }

        StringBuffer sourceName = getSourceNameFromServerURI(*uri);
        TInt sourceID = getSourceIDFromName(sourceName);

        // update and publish new push flags
        switch (sourceID) {
            case SOURCE_CONTACTS: {
                pushVal |= EContactsPush;
                LOG.debug("received push for contacts source");
                valueUpdate = ETrue;

                break;
            }

            case SOURCE_CALENDAR: {
                LOG.debug("received push for calendar source");
                pushVal |= ECalendarPush;
                valueUpdate = ETrue;

                break;
            }

            case SOURCE_NOTES: {
                LOG.debug("received push for notes source");
                pushVal |= ENotesPush;
                valueUpdate = ETrue;

                break;
            }
            
            case SOURCE_MAIL: {
                LOG.debug("received push for mail source");
                pushVal |= EMailPush;
                valueUpdate = ETrue;

                break;
            }

            case -1: default:
                LOG.error("invalid push type received");
                break;
        }
    }

    // publish new push request
    if (valueUpdate) {
        iPushProp.Set(pushVal);
    }
}


void CSMSPushListener::HandleSessionEventL(TMsvSessionEvent aEvent,TAny* aArg1,TAny* aArg2,TAny* aArg3)
{
}

void CSMSPushListener::cleanSMSFromInboxL()
{
    CMsvEntry* folder = iMsvSession->GetEntryL(KMsvGlobalInBoxIndexEntryId);        // Inbox folder
    if (!folder) {
        LOG.error("Folder inbox not found");
        return;
    }
    CleanupStack::PushL(folder);
    
    //
    // Cycle all the entries in the folder, search for SMS push
    //
    if(folder->Count()) {
        
        // useful?
        TMsvSelectionOrdering sortType;
        sortType.SetSorting(EMsvSortByDate);
        folder->SetSortTypeL(sortType);
        
        CMsvEntrySelection* entries = folder->ChildrenL();
        CleanupStack::PushL(entries);
 
        for (TInt i = (folder->Count()-1); i >= 0; i--) {
            
            const TMsvEntry& smsEntry = folder->ChildDataL(entries->At(i));
            if (smsEntry.iMtm == KUidMsgTypeSMS) {
                
                // Read the SMS body
                int msgLength = 0;
                char* msg = getSMSBodyL(smsEntry, &msgLength);
                
                // The SMS header is at least 3 bytes long
                if (msgLength > 3) {
                    
                    // Parse the SMS WSP headers and skip them.
                    int headersLength = parseHeadersWSP(msg, msgLength);
                    if (headersLength > 0) {
                        LOG.debug("SMS Push found in inbox (msg length = %d)", msgLength);
                        char* body     = msg + headersLength;
                        int bodyLength = msgLength - headersLength;

                        //
                        // --- Parse the sync notification pkg#0 and start the sync ---
                        //
                        onPushReceived(body, bodyLength);

                        //
                        // Remove the SMS push from the inbox folder
                        //
                        LOG.debug("Removing binary SMS from inbox");
                        folder->DeleteL(entries->At(i));
                    }
                }
                if (msg) {
                    delete [] msg; 
                    msg = NULL;
                }
            }
        }
        CleanupStack::PopAndDestroy(entries);
    }
    CleanupStack::PopAndDestroy(folder);
}



char* CSMSPushListener::getSMSBodyL(const TMsvEntry& aEntry, int* bodyLength)
{
    CSmsClientMtm* SmsMtm = STATIC_CAST(CSmsClientMtm*, iClientMtmReg->NewMtmL(KUidMsgTypeSMS));
    CleanupStack::PushL(SmsMtm);
    
    SmsMtm->SwitchCurrentEntryL(aEntry.Id());               
    SmsMtm->LoadMessageL();
    
    //
    // Get the message body (can be binary)
    //
    TInt msgLen = SmsMtm->Body().DocumentLength();
    if (msgLen == 0) {
        //LOG.debug("sms body is empty");
        *bodyLength = 0;
        return NULL;
    }
    
    HBufC* message16 = HBufC::NewLC(msgLen);
    TPtr message16Ptr(message16->Des());
    SmsMtm->Body().Extract(message16Ptr, 0, msgLen);

    HBufC8* message8 = HBufC8::NewLC(msgLen);
    message8->Des().Copy(message16->Des());
    //hexDump((char*)message8->Ptr(), message8->Length());
    
    // body is returned new allocated
    char* body = new char[message8->Length()];
    memcpy(body, message8->Ptr(), message8->Length());
    *bodyLength = message8->Length();

    CleanupStack::PopAndDestroy(3);  // SmsMtm, message8, message16

    return body;
}


static void hexDump(char *buf, int len) 
{
    if (LOG.getLevel() < LOG_LEVEL_DEBUG) {
        return;
    }

    char* tmp = new char[len*8 + 3];
    tmp[0] = '[';
    int pos = 1;
    for (int i=0; i<len; i++) {
        sprintf(&tmp[pos], "%02x ", buf[i]);
        pos += 3;
    }
    tmp[pos-1] = ']';
    tmp[pos] = 0;
    //LOG.debug("%s", tmp);
    delete [] tmp;
}
