/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include "uid3.h"

#include "base/Log.h"
#include "base/util/stringUtils.h"
#include "commonDefs.h"
#include "customization.h"
#include "Funambol.hrh"
#include "ClientSettings.h"
#include "FunambolAppUi.h"
#include "About.h"

#include <txtrich.h>    // Richtext
#include <eikrted.h>    // RichText Editor
#include <avkon.mbg>    // Image Ids 
#include <eikenv.h>     
#include "AboutPicture.h"  
#include <aknutils.h>

// include CEikonEnv framework for CEikonEnv::Static()
#include <eikapp.h>
#include <eikappui.h>
#include <eikenv.h>


// MBM name and path
_LIT( KLogoBitmapName, "FunambolSync.mbm" );
_LIT( KResourcePath, "\\resource\\apps\\");

// bitmaps size
static const TInt KLogoBitmapWidth          = 460;
static const TInt KLogoBitmapHeight         = 460;

static const TInt KPoweredByBitmapWidth     = 1178;
static const TInt KPoweredByBitmapHeight    =  221;

// max message buffer size
static const TInt KBufSize = 1024;

CAbout* CAbout::NewL(CFunambolAppUi* aAppUi)
{
    CAbout* self = new (ELeave) CAbout(aAppUi);
    CleanupStack::PushL(self);
    self->ConstructL(aAppUi->ClientRect());
    CleanupStack::Pop(self);
    return self;
}


CAbout::CAbout(CFunambolAppUi* aAppUi) : iAppUi(aAppUi) {}

// Set the Editor flags
void CAbout::SetEditorFlags()
{
    iRtEd->SetAknEditorCase(EAknEditorLowerCase);
    iRtEd->SetAknEditorFlags
        (
        EEikEdwinAllowPictures|
        EAknEditorFlagEnableScrollBars |// Set up the scroller
        EEikEdwinReadOnly | 
        EEikRTedNoTextParsers |
        EEikEdwinAvkonDisableCursor | 
        EEikEdwinAvkonNotEditable |
        EEikEdwinNoAutoSelection

        );
}

void CAbout::LoadBitmapL(const TDesC& aBitMapName, TInt aBitMapId)
{
    TFileName name;
    TBuf< KMaxPath > fullBitMapPath;

    iEikonEnv->FsSession().PrivatePath(name);
    TFileName appFullName = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();
    TParsePtr parse(appFullName);
    TPtrC currentDrive = parse.Drive();

    fullBitMapPath.Copy(currentDrive);
    fullBitMapPath.Append(KResourcePath);
    fullBitMapPath.Append(aBitMapName);

    CFbsBitmap* bitmap;
    // Create and Load the Bitmap
    bitmap= new( ELeave )CFbsBitmap;
    CleanupStack::PushL(bitmap);
    User::LeaveIfError(bitmap->Load(fullBitMapPath, aBitMapId)); 
    iBitmapContainer->AppendL(bitmap);
    CleanupStack::Pop( bitmap ); //container will free bitmap at the destructor
}

//  Call Richtext's InsertL() with the CPicture
void CAbout::InsertPictureL(TInt aIndex, TInt aPos, TInt aWidth, TInt aHeight)
{
    CAboutPicture* picture;
    // Create a CPicture derived class which will draw our image, depending this Size
    picture = new( ELeave )CAboutPicture(TSize(aWidth, aHeight), *(iBitmapContainer->At(aIndex)));
    CleanupStack::PushL(picture);
    // Prepare the Picture header, which will be instered into the Richtext

    TPictureHeader header;
    header.iPicture =TSwizzle<CPicture>(picture);
    
    CRichText* text = iRtEd->RichText();
    text->InsertL(aPos, header);   // insert picture at document start
    CleanupStack::Pop(picture); // Richtext take the ownership of picture

    // bitmap separator
    TBuf<KBufSize> lineBreak;
    lineBreak.Append(CEditableText::ELineBreak);
    text->InsertL(text->DocumentLength(), lineBreak);
    iTextStartPos = text->DocumentLength();
}

// set text attribute (aligment and font color)
void CAbout::SetTextAttrib()
{
    CParaFormat* pf = new (ELeave) CParaFormat();
    CleanupStack::PushL(pf);
    // Set the alignment
    pf->iHorizontalAlignment = CParaFormat::ECenterAlign;
    // Create paragraph formatting mask
    TParaFormatMask mask;
    // This alignment needs to be changed
    mask.SetAttrib(EAttAlignment);
    CRichText* rt = iRtEd->RichText();
    rt->ApplyParaFormatL(pf, mask, 0,  rt->DocumentLength());
    // Apply formatting to global text editor control
    CleanupStack::Pop(); // pf

    delete pf;

    TCharFormat charFormat;
    TCharFormatMask charFormatMask;

    charFormat.iFontPresentation.iTextColor = KRgbBlack;
    charFormatMask.SetAttrib(EAttColor);
    rt->ApplyCharFormatL(charFormat, charFormatMask, 0, rt->DocumentLength());
}

void CAbout::SetupEditorL()
{
    iRtEd->EnableCcpuSupportL(EFalse);
    iRtEd->SetCursorPosL(0, EFalse);
    iRtEd->UpdateScrollBarsL();
    SizeChanged();
}

void CAbout::ConstructL(const TRect& aRect)
{
    CreateWindowL();

    iBitmapContainer = new (ELeave) CArrayFixFlat<CFbsBitmap *>(2); 
    iRtEd = new (ELeave) CEikRichTextEditor;

    SetEditorFlags();

    // load bitmaps
    LoadBitmapL(KLogoBitmapName, EMbmFunambolsyncMainlogo);
    LoadBitmapL(KLogoBitmapName, EMbmFunambolsyncPoweredbylogo);

    iRtEd->ConstructL(this,0,0, EEikEdwinReadOnly | EEikEdwinAvkonDisableCursor);
    iRtEd->SetFocus(ETrue);
        
    // insert logo picture at document start
    InsertPictureL(0, 0, KLogoBitmapWidth, KLogoBitmapHeight);
    SetAboutText();
    SetTextAttrib();

    SetRect(aRect);

    ActivateL();
    SetupEditorL();
}


CAbout::~CAbout()
{
    CloseWindow();

    if (iRtEd) {
        delete iRtEd;
    }

    if (iBitmapContainer) {       
        for (int i=0;i<iBitmapContainer->Count();i++) {
            if (iBitmapContainer->At(i))
                delete iBitmapContainer->At(i);
        }

        delete iBitmapContainer;
    }
}

void CAbout::SizeChanged()
{
    if (iRtEd) {
        TRect rect = Rect();
        TRect ScrollBarRect = iRtEd->ScrollBarFrame()->VerticalScrollBar()->Rect();

        //In 3rd edition CEikRichTextEditor draw the view for the whole rect and
        //the scrollbar doesn't show. That is a reason why CEikRichTextEditor Width() is
        //rect.Width()-ScrollBarRect.Width()
        iRtEd->SetExtent(TPoint(0,0), 
            TSize(rect.Width()-ScrollBarRect.Width(), rect.Height()));
    }
}

TInt CAbout::CountComponentControls() const
{
    return 1; // only one controller (iRtEd) 
}

CCoeControl* CAbout::ComponentControl(TInt aIndex) const
{
    switch ( aIndex )
        {
        case 0:
            return iRtEd;
        default:
            return NULL;
        }
}

void CAbout::Draw(const TRect& aRect) const
{
    CWindowGc& gc = SystemGc();
    gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbWhite);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(aRect);
}

TKeyResponse CAbout::OfferKeyEventL(const TKeyEvent& aKeyEvent,TEventCode aType)
{
    if (iRtEd->IsFocused()) {
        switch (aKeyEvent.iCode) {
            case EKeyDownArrow: 
                iRtEd->MoveCursorL( TCursorPosition::EFPageDown, EFalse );
                return EKeyWasConsumed;

            case EKeyUpArrow:
                iRtEd->MoveCursorL( TCursorPosition::EFPageUp, EFalse );
                return EKeyWasConsumed;

            default:
                iRtEd->OfferKeyEventL(aKeyEvent, aType); 
                return EKeyWasConsumed;
        }
    }
    
    return EKeyWasNotConsumed;
}

void CAbout::SetFontL(TInt aFontId, TInt aStartPos, TInt aLen)
{
    // Get a logical font to base my font on:
    const CFont* logicalFont = AknLayoutUtils::FontFromId(aFontId);
    // Extract font information
    TFontSpec fontspec = logicalFont->FontSpecInTwips();

    TCharFormat charFormat(fontspec.iTypeface.iName, fontspec.iHeight);
    TCharFormatMask charFormatMask;

    charFormatMask.SetAttrib(EAttFontTypeface);
    charFormatMask.SetAttrib(EAttFontHeight);

    CRichText* rtxt= iRtEd->RichText();
    rtxt->ApplyCharFormatL(charFormat, charFormatMask, aStartPos, aLen);

    iRtEd->HandleTextChangedL();
}

    
// set up Funambol About text
void CAbout::SetAboutText()
{
    CRichText* ctxt = iRtEd->RichText();
    TInt pos = 0;
    TInt len = 0;
    TBuf<KBufSize> aboutMsg;
    HBufC* buf = charToNewBuf(APPLICATION_NAME);

    aboutMsg.Append(buf->Des());

    delete buf;

    _LIT(KPreAlphaRelease, "(pre-alpha)");
    _LIT(KAlphaRelease, "(alpha)");
    _LIT(KBetaRelease, "(beta)");
    _LIT(KVersion, "Version ");

    aboutMsg.Append(CEditableText::ELineBreak);
    aboutMsg.Append(KVersion);
    
    buf = stringBufferToNewBuf( ClientSettings::getInstance()->getSwv() );
    aboutMsg.Append(buf->Des());
    delete buf;

    aboutMsg.Append(CEditableText::ELineBreak);

    if (PRE_ALPHA_RELEASE)  { 
        aboutMsg.Append(KPreAlphaRelease);
    } else if (ALPHA_RELEASE) { 
        aboutMsg.Append(KAlphaRelease);
    } else if (BETA_RELEASE)  { 
        aboutMsg.Append(KBetaRelease);
    }

    aboutMsg.Append(CEditableText::ELineBreak);

    pos = ctxt->DocumentLength();
    len = aboutMsg.Length();
    ctxt->InsertL(pos, aboutMsg);
    SetFontL(ELatinBold19, pos, len);

    aboutMsg.Zero();

    if (ENABLE_ABOUT_COPYRIGHT) {
        // read copyright paragraphs and insert into 
        // about text separated by newlines

        TBuf<64> copyright_paragraph;
        CEikonEnv::Static()->ReadResource(copyright_paragraph, R_ABOUT_COPYRIGHT_PARAGRAPH1);
        aboutMsg.Append(copyright_paragraph);
        aboutMsg.Append(CEditableText::ELineBreak);
       
        copyright_paragraph.Zero();

        CEikonEnv::Static()->ReadResource(copyright_paragraph, R_ABOUT_COPYRIGHT_PARAGRAPH2);
        aboutMsg.Append(copyright_paragraph);
        aboutMsg.Append(CEditableText::ELineBreak);

        copyright_paragraph.Zero();

        CEikonEnv::Static()->ReadResource(copyright_paragraph, R_ABOUT_COPYRIGHT_PARAGRAPH3);
        aboutMsg.Append(copyright_paragraph);
        aboutMsg.Append(CEditableText::ELineBreak);
        aboutMsg.Append(CEditableText::ELineBreak);
    }    
        
    if (ENABLE_ABOUT_MAIN_WEB_SITE) {
        TBuf<64> mainWebSite;
        CEikonEnv::Static()->ReadResource(mainWebSite, R_ABOUT_MAIN_WEB_SITE);
        aboutMsg.Append(mainWebSite);
    }   

    aboutMsg.Append(CEditableText::ELineBreak);
    aboutMsg.Append(CEditableText::ELineBreak);

    pos = ctxt->DocumentLength();
    len = aboutMsg.Length();
    ctxt->InsertL(pos, aboutMsg);
    SetFontL(ELatinBold16, pos, len);

    if (ENABLE_ABOUT_LICENSE) {
        // read license paragraphs and insert into 
        // about text separated by newlines

        aboutMsg.Zero();
        TBuf<512> license_paragraph;
        CEikonEnv::Static()->ReadResource(license_paragraph, R_ABOUT_LICENSE_PARAGRAPH1);
        aboutMsg.Append(license_paragraph);
        aboutMsg.Append(CEditableText::ELineBreak);
        aboutMsg.Append(CEditableText::ELineBreak);

        license_paragraph.Zero();
 
        CEikonEnv::Static()->ReadResource(license_paragraph, R_ABOUT_LICENSE_PARAGRAPH2);
        aboutMsg.Append(license_paragraph);
        aboutMsg.Append(CEditableText::ELineBreak);
        aboutMsg.Append(CEditableText::ELineBreak);

        license_paragraph.Zero();
 
        CEikonEnv::Static()->ReadResource(license_paragraph, R_ABOUT_LICENSE_PARAGRAPH3);
        aboutMsg.Append(license_paragraph);
        aboutMsg.Append(CEditableText::ELineBreak);

        pos = ctxt->DocumentLength();
        len = aboutMsg.Length();
        ctxt->InsertL(pos, aboutMsg);
        SetFontL(ELatinBold13, pos, len);
    }

    if (ENABLE_ABOUT_POWERED_BY) {
        aboutMsg.Append(CEditableText::ELineBreak);
        aboutMsg.Append(CEditableText::ELineBreak);
        pos = ctxt->DocumentLength();
        InsertPictureL(1, pos, KPoweredByBitmapWidth, KPoweredByBitmapHeight);
        aboutMsg.Append(CEditableText::ELineBreak);
        aboutMsg.Append(CEditableText::ELineBreak); // to ensure text scrolls correctly

        aboutMsg.Zero();

        if (ENABLE_ABOUT_POWERED_BY_WEB_SITE) {
            TBuf<64> poweredByWebSite;
            CEikonEnv::Static()->ReadResource(poweredByWebSite, R_ABOUT_POWERED_WEB_SITE);
            aboutMsg.Append(CEditableText::ELineBreak);
            aboutMsg.Append(poweredByWebSite);
        }
    
        aboutMsg.Append(CEditableText::ELineBreak);
        aboutMsg.Append(CEditableText::ELineBreak); // to ensure text scrolls correctly

        pos = ctxt->DocumentLength();
        len = aboutMsg.Length();
        ctxt->InsertL(pos, aboutMsg);
        SetFontL(ELatinBold13, pos, len);
    }
}
