/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <barsread.h>
#include "uid3.h"

#include "base/Log.h"
#include "base/util/stringUtils.h"
#include "push/FThread.h"

#include "commonDefs.h"
#include "customization.h"
#include "Funambol.hrh"
#include "ClientSettings.h"
#include "FunambolAppUi.h"
#include "ProgressDialog.h"

#include "AccountSettingsDialog.h"

using namespace Funambol;

int CAccountSettingsDialog::iEnableServerUrl = ENABLE_SERVER_URL;


/////////////////////////////////////////////////////////////////
class ConfigSyncThread : public FThread {
private:
    ConfigSyncThread() {}

public:
    ConfigSyncThread(CFunambolAppUi* appUi) : appUi(appUi) {
    }

    void run() {
        LOG.debug("Firing synchronization");
        appUi->StartSync(SOURCE_CONFIG, ESyncTriggeredManual);
        LOG.debug("Terminated");
    }

private:
    CFunambolAppUi* appUi;
};
/////////////////////////////////////////////////////////////////




CAccountSettingsDialog* CAccountSettingsDialog::NewL(TInt aMenuTitleResourceId, 
                                                     CFunambolAppUi* appUi,
                                                     TBool& aCancelled)
{
    CAccountSettingsDialog* self = new (ELeave)CAccountSettingsDialog(appUi, aCancelled);
    CleanupStack::PushL(self);
    self->ConstructL(aMenuTitleResourceId);
    CleanupStack::Pop(self);
    return self;
}

CAccountSettingsDialog::CAccountSettingsDialog(CFunambolAppUi* appUi, TBool& aCancelled)
                                               : iAppUI(appUi),
                                                 iCancelled(aCancelled),
                                                 iSyncConfigEnded(false),
                                                 iUiLocked(EFalse),
                                                 iGoToMainScreen(false),
                                                 iProgressDlg(NULL)
                                                 
{
    iUsername    = "";
    iPassword    = "";
    iSyncURL     = "";
    iSettingList = NULL;
    iCounterServerUrl = 0;
    iEnableServerUrl = ENABLE_SERVER_URL;
}


CAccountSettingsDialog::~CAccountSettingsDialog()
{
    //LOG.debug("destructor");
    if (iSettingList) {
        delete iSettingList;
        iSettingList = NULL;
    }
}


void CAccountSettingsDialog::PreLayoutDynInitL()
{
    //LOG.debug("CAccountSettingsDialog::PreLayoutDynInitL");
    
    if (!iSettingList) // if we haven't already created iSettingList
    {
        // construct control and set parent
        iSettingList = new (ELeave) CCommonOptionsSettingList();
        iSettingList->SetMopParent(this);
    
        // Loads data into the setting list.
        // NOTE: we must load values into the settingList BEFORE constructing it,
        //       so it will be constructed with the right default values.
        load();

        // construct control based on resource definition
        TResourceReader reader;

        // CreateResourceReaderLC will allocate a buffer to be used by
        // the TResourceReader. This buffer is pushed onto the cleanup
        // stack - not the TResourceReader itself
        iCoeEnv->CreateResourceReaderLC(reader, R_ACCOUNT_SETTING_LIST);
        iSettingList->ConstructFromResourceL(reader);


        // Clean up the buffer allocated above, NOT the reader itself.
        // Cannot use expected item overload of PopAndDestroy() as buffer 
        // is inaccessible. 
        CleanupStack::PopAndDestroy();
        
        // Removes the vertical scrollbar
        iSettingList->initScrollbar();
        
        // Show the "*" symbol near these items.
        iSettingList->SetMandatoryItem(ESettingsListUsername);
        iSettingList->SetMandatoryItem(ESettingsListPassword);
        iSettingList->SetMandatoryItem(ESettingsListSyncUrl);
        
        // Show/Hide the ServerURL item.
        if (!iEnableServerUrl) {
            iSettingList->HideItem(ESettingsListSyncUrl, ETrue);
        }

        // Activate the settings list
        iSettingList->ActivateL();
    }
    
    CAknDialog::PreLayoutDynInitL();
}


TKeyResponse CAccountSettingsDialog::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType != EEventKey) {
        checkToEnableServerUrl(aKeyEvent, aType);
        return EKeyWasNotConsumed;
    }
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3)          // ???????? why this code ????????
    {
        iSettingList->EditCurrentItemL();
        return EKeyWasConsumed;
    }
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to setting list
        case EKeyDownArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}


void CAccountSettingsDialog::ProcessCommandL( TInt aCommand )
{
    //LOG.debug("CAccountSettingsDialog::ProcessCommandL - event received: %d", aCommand);
    switch( aCommand )
    {
        case EAccountSettingsChange:      
        {
            if (MenuShowing()) { HideMenu(); }
            iSettingList->EditCurrentItemL();
            break;
        }
        case EAknSoftkeyCancel:
        case EAknSoftkeyExit:
        {
            // Forward the command to OkToExitL, so that we can
            // exit this dialog.
            TryExitL(aCommand);     // will call OkToExitL()
            break;
        }

        default: 
        {
            // Other commands forwarded to CAknDialog.
            CAknDialog::ProcessCommandL(aCommand);
        }
    }
}

void CAccountSettingsDialog::CheckCredentialsEnded(bool aGoToMainScreen)
{
    iSyncConfigEnded = true;
    iGoToMainScreen  = aGoToMainScreen;
    TryExitL(EAknSoftkeyOk);
}

TBool CAccountSettingsDialog::OkToExitL(TInt aButtonId)
{
    //LOG.debug("***** CAccountSettingsDialog::OkToExitL: button %d", aButtonId);
    
    if (iAppUI->CheckSyncInProgress()) {
        // Catch all commands during config sync!
        //LOG.debug("***** Catched button %d (lock = %s)", aButtonId, iUiLocked? "true":"false");
        return EFalse;
    }

    // Returning ETrue causes the form to exit
    switch (aButtonId)
    {
        case EAknSoftkeyCancel:
        case EAknSoftkeyExit:
        {
            // User cancelled: leave the dialog.
            // FunambolAppUI will chose the proper action to execute.
            LOG.debug("Exiting AccountSettings dialog");
            iCancelled = true;
            return ETrue;
        }
        case EAknSoftkeyOk:
        case EAknSoftkeyBack:
        case EAknSoftkeyDone:
        {
            if (iSyncConfigEnded) {
                if (!iGoToMainScreen) {
                    // Something wrong: save old values again into config.
                    restorePreviousSettings();
                }
                
                // Close the wait note, show again the buttons.
                lockUI(EFalse);
                
                iCancelled = false;
                iSyncConfigEnded = false;
                return iGoToMainScreen;   // ETrue will leave the dialog
            } 
            else {
                //
                // Save the settings from UI dialog into configuration.
                // Starts the config sync (non blocking) if necessary.
                //
                return save();
            }
        }
        default:
        {
            // Other commands forwarded to CAknDialog.
            return CAknDialog::OkToExitL(aButtonId);
        }
    }
    return EFalse;
}



void CAccountSettingsDialog::load()
{
    LOG.info("Reading Account data...");
    ClientSettings* config = ClientSettings::getInstance();
    config->open();

    // Read values from config
    iUsername = config->getAccessConfig().getUsername();
    iPassword = config->getAccessConfig().getPassword();
    iSyncURL  = config->getAccessConfig().getSyncURL();
    
    // Set values into the settinglist
    iSettingList->setUsername(iUsername);
    iSettingList->setPassword(iPassword);
    iSettingList->setSyncURL(iSyncURL);
    
    config->close();
}

TBool CAccountSettingsDialog::save()
{
    ClientSettings* config = ClientSettings::getInstance();
    bool saveConfig = true;
    
    //
    // Read values inserted by the user
    //
    StringBuffer newUsername = iSettingList->getUsername();
    StringBuffer newPassword = iSettingList->getPassword();
    StringBuffer newSyncURL(iSyncURL);
    if (iEnableServerUrl) { newSyncURL = iSettingList->getSyncURL(); } 
    
    //
    // Check input validity
    //
    if (ENABLE_CREDENTIALS_VALIDITY_CHECK) 
    {
        if (newUsername.empty()) {
            goto badCredentials;
        }
        if (ENABLE_AT_VALIDITY_CHECK) {
            if (newUsername.find("@") == StringBuffer::npos) {
                goto badCredentials;
            }
        }
        if (newPassword.empty()) {
            goto badCredentials;
        }
        
        if (iEnableServerUrl) 
        { 
            if (newSyncURL.empty()) {
                goto badCredentials;
            }
            newSyncURL.lowerCase();
            
            // Add "http://" at the beginning, if missing.
            if ( (newSyncURL.find("http://")  == StringBuffer::npos) && (newSyncURL.find("https://") == StringBuffer::npos) ) {
                StringBuffer tmp("http://");
                tmp.append(newSyncURL);
                newSyncURL = tmp;
            }
        }
        
        // Add port number, if missing (FIX for S60 3ed FP1).
        addPortIfMissing(newSyncURL);
    }

    //
    // Check if values changed
    //
    if ( (newUsername == iUsername) && (newPassword == iPassword) && (newSyncURL == iSyncURL) ) 
    {
        LOG.info("Account data unchanged.");
        saveConfig = false;
        
        if (config->getPendingSyncConfig() == false) {
            LOG.debug("Config sync not necessary: return to main screen.");
            iCancelled = false;
            return ETrue;       // returns to main screen
        } 
        else {
           LOG.debug("Config sync is pending");
        }
    }

    
    // Hide the buttons and show a wait note while the sync is in progress.
    // So during the whole saving time, the user has a wait note.
    lockUI(ETrue);
    
    
    //
    // Store values to config.
    //
    if (saveConfig) 
    {
        LOG.info("Saving Account data...");
        config->open();
    
        LOG.debug("Setting username to: %s", newUsername.c_str());
        config->setUsername(newUsername);
    
        LOG.debug("Setting password to **** (hidden)");
        config->setPassword(newPassword);
        
        LOG.debug("Setting server URL to %s", newSyncURL.c_str());
        config->setSyncURL(newSyncURL);
    
        config->save();
        config->close();
        
        // Don't update internal values: the config sync could not finish
        // correctly, in that case we keep old values.
    }
    

    //
    // Start the config sync (non blocking, with timeout).
    //
    iAppUI->checkCredentials();
    return EFalse;      // stay in this dialog
    
    
badCredentials:

    iAppUI->DisplayMessage(ERR_BAD_INPUT_CREDENTIALS, EErrorLevel);
    return EFalse;      // stay in this dialog
}


void CAccountSettingsDialog::lockUI(const TBool aEnable)
{
    CEikButtonGroupContainer* cba = CEikButtonGroupContainer::Current();
    cba->SetDimmed(aEnable);
    cba->DrawNow();

    if (aEnable) {
        iProgressDlg = new ProgressDialog(false);
        iProgressDlg->StartWaitNoteL();
        TBuf<128> noteText;
        iEikonEnv->ReadResourceL(noteText, R_STRING_CHECKING_CREDENTIALS);
        iProgressDlg->UpdateProcessL(0, noteText);
    }
    else {
        delete iProgressDlg;
        iProgressDlg = NULL;
    }
    
    iUiLocked = aEnable;
}


void CAccountSettingsDialog::restorePreviousSettings()
{
    LOG.info("Restoring previous Account data...");
    ClientSettings* config = ClientSettings::getInstance();
    config->open();

    LOG.debug("Setting username to: %s", iUsername.c_str());
    config->setUsername(iUsername);

    iPassword = "";
    LOG.debug("Setting password to \"\" (empty)");
    config->setPassword(iPassword);
    
    LOG.debug("Setting server URL to %s", iSyncURL.c_str());
    config->setSyncURL(iSyncURL);

    config->save();
    config->close();
}


void CAccountSettingsDialog::checkToEnableServerUrl(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType == EEventKeyDown) {
        if (aKeyEvent.iScanCode == '6') {
            iCounterServerUrl ++;
            if (iCounterServerUrl == 6) {
                iEnableServerUrl = 1;
                iSettingList->HideItem(ESettingsListSyncUrl, EFalse);
            }
        } else {
            iCounterServerUrl = 0;
        }
    }
}



bool CAccountSettingsDialog::addPortIfMissing(StringBuffer& syncUrl) 
{
    bool ret = false;
    size_t start, firstSlash, firstColon, urlLength;
    urlLength = syncUrl.length();
    
    // Default port for HTTPS is "443"
    StringBuffer defaultPort("80");
    if (syncUrl.find("https") != StringBuffer::npos) {
        defaultPort = "443";
    }
    
    // Get the starting position from syncUrl: e.g. "http://<hostName>:8080/funambol/ds"    
    start = syncUrl.find("://", 0);
    if (start != StringBuffer::npos) {
        start += 3;
    } else {
        // "<hostName>:8080/funambol/ds"  
        start = 0;
    }
    
    firstSlash = syncUrl.find("/", start);       // stop if "/" found
    firstColon = syncUrl.find(":", start);       // stop if ":" found
    //LOG.debug("url = %s, start = %d, firstSlash = %d, firstColon = %d", syncUrl.c_str(), start, firstSlash, firstColon);
    
    if (firstSlash == StringBuffer::npos && firstColon == StringBuffer::npos) {
        // "hostname"
        syncUrl += ":";
        syncUrl += defaultPort;
        ret = true;
    } 
    else if (firstSlash != StringBuffer::npos && firstColon == StringBuffer::npos) {
        // "hostname/funambol"
        StringBuffer newUrl(syncUrl.substr(0, firstSlash));
        newUrl += ":";
        newUrl += defaultPort;
        newUrl += syncUrl.substr(firstSlash, urlLength - firstSlash);
        syncUrl = newUrl;
        ret = true;
    } 
    else if (firstSlash == StringBuffer::npos && firstColon != StringBuffer::npos) {
        // "hostname:8080"
        if (urlLength > firstColon+1) {
            // Not empty port: we trust it's a good value.
            return false;
        }
        // "hostname:"
        syncUrl += defaultPort;
        ret = true;
    } 
    else if (firstSlash > firstColon) {
        // "hostname:8080/funambol"
        if (firstSlash > firstColon+1) {
            // Not empty port: we trust it's a good value.
            return false;
        }
        // "hostname:/funambol"
        StringBuffer newUrl(syncUrl.substr(0, firstSlash));
        newUrl += defaultPort;
        newUrl += syncUrl.substr(firstSlash, urlLength - firstSlash);
        syncUrl = newUrl;
        ret = true;
    }
    
    LOG.info("synchronization URL is '%s'", syncUrl.c_str());
    return ret;
}
