/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/SymbianLog.h"
#include "base/FConnection.h"
#include "CommonOptionsSettingList.h"
#include "FunambolAppUi.h"
#include "Funambol.hrh"
#include "utils.h"

using namespace Funambol;


const TBuf<1> CCommonOptionsSettingList::emptyText = _L("");


CCommonOptionsSettingList::CCommonOptionsSettingList() : CAknSettingItemList()
{
    iSyncEnabled  = ETrue;
    iAutostart    = ETrue;
    iSyncMode     = ESyncModeNormal;
    iSyncUsing    = ESyncUsingPush;
    iSyncInterval = 0;
    iAPNSelection = 0;
    iLogLevel     = LOG_LEVEL_INFO;
    iMemoryInUse  = EMemoryCardStorage;
    iIncludeOlderPictures = 0;
    iIncludeOlderPictures = 0;
    
    iUsername.Copy(_L(""));
    iSyncURL.Copy(_L(""));
    iPassword.Copy(_L(""));
    
    iCleanupEmailEnabled    = EFalse;
    iCleanupContactEnabled  = EFalse;
    iCleanupCalendarEnabled = EFalse;
    iCleanupNotesEnabled    = EFalse;
    iCleanupPhotosEnabled   = EFalse;
}

void CCommonOptionsSettingList::initScrollbar()
{
    ListBox()->CreateScrollBarFrameL(ETrue);
    ListBox()->ScrollBarFrame()->SetScrollBarVisibilityL( CEikScrollBarFrame::EAuto,
                                                          CEikScrollBarFrame::EOff ); 
}

CAknSettingItem* CCommonOptionsSettingList::CreateSettingItemL(TInt aIdentifier)
{
    CAknSettingItem* settingItem = NULL;
    switch (aIdentifier)
    {
        // ------- Account settings dialog -------
        case ESettingsListUsername:
            settingItem = new (ELeave) CAknTextSettingItem(aIdentifier, iUsername);
            break;
        case ESettingsListSyncUrl:
            settingItem = new (ELeave) CAknTextSettingItem(aIdentifier, iSyncURL);
            break;
        case ESettingsListPassword:
            settingItem = new (ELeave) CAknPasswordSettingItem(aIdentifier, 
                                                               CAknPasswordSettingItem::EAlpha, 
                                                               iPassword);
            break;
          
        // ------- Sub-options dialogs -------
        case ESettingsListSyncEnable:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iSyncEnabled);
            break;
        case ESettingsListSyncMode:
            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iSyncMode);
            break;
        case ESettingsListSyncUsing:

            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iSyncUsing);
            break;
        case ESettingsListSyncInterval:
            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iSyncInterval);
            break;
        case ESettingsListAutostart:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iAutostart);
            break;  
        case ESettingsListAPNSelection:
            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iAPNSelection);
            break;
        case ESettingsListLogLevel:
            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iLogLevel);
            break;
        case ESettingListMemoryInUse: 
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iMemoryInUse);
            break;
        case ESettingListMassStorageMemoryInUse: 
            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iMemoryInUse);
            break;
        case ESettingListIncludeOlderPictures:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iIncludeOlderPictures);
            break;

        // ------ Cleanup dialogs ----------
        case ESettingsListCleanupEmail:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iCleanupEmailEnabled);
            break;
        case ESettingsListCleanupContact:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iCleanupContactEnabled);
            break;
        case ESettingsListCleanupCalendar:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iCleanupCalendarEnabled);
            break;
        case ESettingsListCleanupNotes:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iCleanupNotesEnabled);
            break;
        case ESettingsListCleanupPhotos:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iCleanupPhotosEnabled);
            break;
    }
    return settingItem;
}



void CCommonOptionsSettingList::ConstructAPNList(HBufC* defaultAPNText)
{
    //LOG.debug("CCommonOptionsSettingList::ConstructAPNList");
    
    // Find the APN-Selection item.
    CAknEnumeratedTextPopupSettingItem* apnPopup;
    apnPopup = (CAknEnumeratedTextPopupSettingItem*)findItem(ESettingsListAPNSelection); 
    if (!apnPopup) {
        LOG.error("APN popup NULL, exit");
        return;
    }
    
    // Get the list of current IAP names from FConnection.
    FConnection* connection = FConnection::getInstance();
    if (!connection) {
        LOG.error("Cannot open FConnection");
        return;
    }
    RArray<HBufC*> iapNames = connection->GetAllIAPNames();
    
    
    // Set the array of values
    CArrayPtr<CAknEnumeratedText>* apnValues = apnPopup->EnumeratedTextArray();
    apnValues->ResetAndDestroy();
    
    TInt enumValue = 0;
    if (defaultAPNText) {
        CAknEnumeratedText* text = new CAknEnumeratedText(enumValue++, defaultAPNText);
        apnValues->AppendL(text);
    }
    for (int i=0; i<iapNames.Count(); i++) {
        HBufC* tmp = iapNames[i]->Alloc();
        CAknEnumeratedText* text = new CAknEnumeratedText(enumValue++, tmp);
        apnValues->AppendL(text);
    }

    // Tell the control to load the value in
    apnPopup->LoadL();
    //LOG.debug("Number of APN items = %d", apnValues->Count());
    
    iapNames.Close();
}


void CCommonOptionsSettingList::EditCurrentItemL()
{
    TInt current = ListBox()->CurrentItemIndex();
    TInt index = GetGlobalItemsIndex(current);
    //LOG.debug("EditCurrentItemL - current index = %d", current);
    //LOG.debug("EditCurrentItemL - global index = %d", index);

    // Invoke EditItemL on the current item: shows the popup dialog.
    EditItemL(index, ETrue);
    
    // FIX: refresh the password text (otherwise 1st time is not updated).
    CAknSettingItem* settingItem = SettingItemArray()->At(index);
    if (settingItem && settingItem->Identifier() == ESettingsListPassword) {
        settingItem->UpdateListBoxTextL();
    }

    // Force to store variables for all items.
    StoreSettingsL();
}


TInt CCommonOptionsSettingList::GetGlobalItemsIndex(const TInt index)
{
    TInt visibleIndex = -1;
    TInt i = 0;
    for (i=0; i<SettingItemArray()->Count(); i++)
    {
        CAknSettingItem* settingItem = SettingItemArray()->At(i);
        if (settingItem && !(settingItem->IsHidden())) {
            // Found an item that is not hidden.
            visibleIndex ++;
        }
        if (visibleIndex == index) {
            // Reached the index of our item -> break.
            break;
        }
    }
    return i;
}



void CCommonOptionsSettingList::HideItem(const TSettingItems aItemID, const TBool aHide)
{
    CAknSettingItem* settingItem = findItem(aItemID);
    if (settingItem) {
        settingItem->SetHidden(aHide);
        
        TRAPD(err, HandleChangeInItemArrayOrVisibilityL();)
        if (err) {
            LOG.error("HandleChangeInItemArrayOrVisibilityL leave with %d", err);
        }
    }
}

void CCommonOptionsSettingList::SetMandatoryItem(const TSettingItems aItemID)
{
    CAknSettingItem* settingItem = findItem(aItemID);
    if (settingItem) {
        settingItem->SetCompulsoryIndTextL(_L("*"));
    }
}


void CCommonOptionsSettingList::RefreshItem(const TSettingItems aItemID)
{
    CAknSettingItem* settingItem = findItem(aItemID);
    if (settingItem) {
        settingItem->UpdateListBoxTextL();
    }
}

const TDesC& CCommonOptionsSettingList::GetItemText(const TSettingItems aItemID)
{
    CAknSettingItem* settingItem = findItem(aItemID);
    if (settingItem) {
        return settingItem->SettingTextL();
    }
    return emptyText;
}


CAknSettingItem* CCommonOptionsSettingList::findItem(const TSettingItems aItemID) 
{
    TInt numItems = SettingItemArray()->Count();
    for (TInt i=0; i<numItems; i++) 
    {
        CAknSettingItem* settingItem = SettingItemArray()->At(i);
        if (settingItem && settingItem->Identifier() == aItemID) 
        {
            return settingItem;
        }
    }
    return NULL;    // Not found
}

TInt CCommonOptionsSettingList::getCurrentItemID()
{
    TInt current = ListBox()->CurrentItemIndex();
    TInt index   = GetGlobalItemsIndex(current);
    
    CAknSettingItem* settingItem = SettingItemArray()->At(index);
    if (settingItem) {
        return settingItem->Identifier();
    }
    return KErrNotFound;    // Not found (should not happen)
}



void CCommonOptionsSettingList::setUsername(const StringBuffer& value)
{
    RBuf tmp;
    tmp.Assign(stringBufferToNewBuf(value));
    iUsername.Copy(tmp);
    tmp.Close();
}
void CCommonOptionsSettingList::setPassword(const StringBuffer& value)
{
    RBuf tmp;
    tmp.Assign(stringBufferToNewBuf(value));
    iPassword.Copy(tmp);
    tmp.Close();
}
void CCommonOptionsSettingList::setSyncURL(const StringBuffer& value)
{
    RBuf tmp;
    tmp.Assign(stringBufferToNewBuf(value));
    iSyncURL.Copy(tmp);
    tmp.Close();
}


// ------------------- CEasyBinaryPopupSettingItem methods -------------------

void CEasyBinaryPopupSettingItem::EditItemL( TBool aCalledFromMenu )
{
    if (InternalValue() == 0) 
    {
        SetInternalValue(1);
        SetSelectedIndex(1);
        SetExternalValue(1);
    }
    else 
    {
        SetInternalValue(0);
        SetSelectedIndex(0);
        SetExternalValue(0);
    }
    
    UpdateListBoxTextL();
}


