/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/SymbianLog.h"
#include "base/util/symbianUtils.h"
#include "DefaultSymbianConfigFactory.h"
#include "commonDefs.h"
#include "customization.h"

using namespace Funambol;


DefaultSymbianConfigFactory::DefaultSymbianConfigFactory() : DefaultConfigFactory() {
}

DefaultSymbianConfigFactory::~DefaultSymbianConfigFactory() {
}


AccessConfig* DefaultSymbianConfigFactory::getAccessConfig() {

    AccessConfig* ac = new AccessConfig();

    // NOTE: make sure to escape any "&" or "<" char here.
    ac->setUsername             ("");
    ac->setPassword             ("");
    ac->setFirstTimeSyncMode    (SYNC_NONE);
    ac->setUseProxy             (false);
    ac->setProxyHost            ("");
    ac->setProxyPort            (8080);
    ac->setProxyUsername        ("");
    ac->setProxyPassword        ("");
    ac->setSyncURL              (SYNC_URI);
    ac->setBeginSync            (0);
    ac->setEndSync              (0);
    ac->setServerAuthRequired   (false);
    ac->setClientAuthType       ("syncml:auth-basic");
    ac->setServerAuthType       ("syncml:auth-basic");
    ac->setServerPWD            ("funambol");
    ac->setServerID             ("funambol");
    ac->setServerNonce          ("");
    ac->setClientNonce          ("");
    ac->setMaxMsgSize           (MAX_SYNCML_MSG_SIZE);      // 64kB
    ac->setReadBufferSize       (0);
    ac->setUserAgent            (APPLICATION_NAME);         // Will be replaced in ClientSettings::createDefaultConfig()
    ac->setCheckConn            (true);
    ac->setResponseTimeout      (600);                      // [seconds] timeout on server = 10 min

    return ac;
}


DeviceConfig* DefaultSymbianConfigFactory::getDeviceConfig() {

    DeviceConfig* dc = new DeviceConfig();
    
    // NOTE: make sure to escape any "&" or "<" char here.
    dc->setMan                  (MANUFACTURER_NAME);
    dc->setMod                  ("");
    dc->setOem                  ("");
    dc->setFwv                  ("");
    dc->setSwv                  ("");                       // Will be replaced in ClientSettings::createDefaultConfig()
    dc->setHwv                  ("");
    dc->setDevID                ("");                       // Will be replaced in ClientSettings::createDefaultConfig()
    dc->setDevType              ("phone");
    dc->setDsV                  ("");
    dc->setUtc                  (true);
    dc->setLoSupport            (true);                     // ?
    dc->setNocSupport           (false);                    // ?
    dc->setLogLevel             ((LogLevel)DEFAULT_LOG_LEVEL);
    dc->setMaxObjSize           (0);
    dc->setDevInfHash           ("");
    dc->setSendDevInfo          (false);                    // don't send devinfo    

    return dc;
}



SyncSourceConfig* DefaultSymbianConfigFactory::getSyncSourceConfig(const char* name) {

    SyncSourceConfig* sc = new SyncSourceConfig();

    // Common props
    sc->setName                 (name);
    sc->setSync                 ("two-way");
    sc->setEncoding             ("bin");
    sc->setEncryption           ("");
    sc->setIsEnabled            (true);
    
    // *** since 7.1.12 ***
    // '0' would force a slow sync: we don't want to force a slow by default!
    // In case the syncML profile SYNC_PROFILE_NAME does not exist, then we'll force a slow.
    sc->setLast                 (1);

    if (!strcmp(name, SOURCE_CONTACTS_NAME))
    {
        sc->setSyncModes        ("slow,two-way,one-way-server,one-way-client");
        sc->setURI              (CARD_REMOTE_URI);
        sc->setType             ("text/x-vcard");
        sc->setVersion          ("2.1");
        sc->setSupportedTypes   ("text/x-vcard:2.1");
    }
    else if (!strcmp(name, SOURCE_CALENDAR_NAME))
    {
        sc->setSyncModes        ("slow,two-way,one-way-server,one-way-client");
        sc->setURI              (CAL_REMOTE_URI);
        sc->setType             ("text/x-vcalendar");
        sc->setVersion          ("1.0");
        sc->setSupportedTypes   ("text/x-vcalendar:1.0");
    }
    else if (!strcmp(name, SOURCE_MAIL_NAME))
    {
        sc->setSyncModes        ("slow,two-way");
        sc->setURI              ("mail");
        sc->setType             ("text/plain");
        sc->setVersion          ("");
        sc->setSupportedTypes   ("text/plain");
    }
    else if (!strcmp(name, SOURCE_NOTES_NAME))
    {
        sc->setSyncModes        ("two-way,one-way-server,one-way-client");
        sc->setURI              (NOTES_REMOTE_URI);
        sc->setType             ("text/plain");
        sc->setVersion          ("");
        sc->setSupportedTypes   ("text/plain");
    }
    else if (!strcmp(name, SOURCE_PHOTOS_NAME))
    {
    	sc->setSyncModes        ("slow,smart-one-way-from-client,refresh-from-client");
    	sc->setURI              (PICTURES_REMOTE_URI);
    	sc->setVersion          ("");
    	sc->setSync             (DEFAULT_PICTURES_SYNC_MODE);    // Special syncmode for pictures
    	
    	if (USE_PICTURES_RAW_FILES) {
    	    sc->setType          ("application/*");
    	    sc->setSupportedTypes("application/*");
    	    sc->setEncoding      ("b64");
    	}
    	else {
            sc->setType          ("application/vnd.omads-file+xml");
            sc->setSupportedTypes("application/vnd.omads-file+xml"); 
            sc->setEncoding      ("bin");
    	}
    }
    else if (!strcmp(name, SOURCE_CONFIG_NAME))
    {
        sc->setSyncModes        ("slow,two-way,refresh");
        sc->setURI              ("configuration");
        sc->setType             ("text/plain");
        sc->setVersion          ("");
        sc->setSupportedTypes   ("text/plain");
    }

    return sc;
}

