/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

/*
* ============================================================================
*  Name     : CImapEmailEngine from InternetEmailEngine.cpp
*  Part of  : InternetEmail
*  Created  : 25.10.2005 by Forum Nokia
*  Implementation notes:
*     This is the main class, the engine, of this application.
*      class itself is basic state handler that derives from CActive
*      and implemenents MMsvSessionObserver interface.
*     Initial content was not generated by S60 AppWizard.
*  Version  : 1.1
*  Copyright: Nokia Corporation 2004
* ============================================================================
*/

#include <badesca.h>
#include <mncnnotification.h>

#include "base/Log.h"
#include "base/util/stringUtils.h"
#include "base/util/symbianUtils.h"

#include "customization.h"
#include "FunambolAppUi.h"
#include "ClientSettings.h"
#include "ImapEmailEngine.h"

using namespace Funambol;


CImapEmailEngine* CImapEmailEngine::NewL(CSyncEngine* syncEngine,
                                         CFunambolAppUi* appUi,
                                         TInt sourceId)
{
    CImapEmailEngine* engine = new(ELeave) CImapEmailEngine(syncEngine, appUi, sourceId);
    CleanupStack::PushL(engine);
    engine->ConstructL();
    CleanupStack::Pop(engine);
    return engine;
}

CImapEmailEngine::CImapEmailEngine(CSyncEngine* syncEngine, CFunambolAppUi* appUi,
                                   TInt sourceId)
    : CActive(CActive::EPriorityStandard),
      appUi(appUi),
      sourceId(sourceId),
      syncEngine(syncEngine)
{
    // add this object to the active scheduler
    CActiveScheduler::Add(this);
}

CImapEmailEngine::~CImapEmailEngine()
{
    if( IsActive() ) {
        Cancel();
    }
    if(iMsvOp) {
        iMsvOp->Cancel();
    }
    delete iMsvOp;
    iMsvOp = NULL;

    delete iEntry;
    iEntry = NULL;

    delete iMtm;
    iMtm = NULL;

    if (iRemoteEntries) {
        iRemoteEntries->Reset();
    }
    delete iRemoteEntries;
    iRemoteEntries = NULL;

    delete iUiReg;
    iUiReg = NULL;

    delete iClientReg;
    iClientReg = NULL;

    delete iMsvSession; //Session must be deleted last
    iMsvSession = NULL;
    
    iEmailIDs.Close();
}

void CImapEmailEngine::ConstructL()
{
    // create and open a session towards messaging
    iMsvSession=CMsvSession::OpenSyncL(*this);

    // Default protocol values
    iProtocolUid=KUidMsgTypeSMTP;
    iHasImapInbox=EFalse;

    //for message handling
    iServiceId=KMsvRootIndexEntryId;
    iId=KMsvRootIndexEntryId;
    iRemoteEntries = new(ELeave) CMsvEntrySelection();

    // other member initalization
    iState=EInitialising;

    iClientReg = CClientMtmRegistry::NewL( *iMsvSession );
    iUiReg = CMtmUiRegistry::NewL( *iMsvSession );

    iLastErrorCode = 0;
    iSyncInProgress = EFalse;

    iMtm = NULL;
}


void CImapEmailEngine::RunL()
{
    TBuf8<1> null;

    switch( iState ) // separation between states
    {

        // ------------------------------------
        //    STATE: EConnecting
        //   usually opens internet connection
        // ------------------------------------
        case EConnecting:
        {
        	
            //As KIMAP4MTMConnect, but, after connection to
            //the IMAP service, starts a background synchronisation.
            //The call completes when the connection occurs
            //and the synchronisation starts.
            LOG.debug("Connecting to IMAP Server...");
            appUi->UpdateSourceStatus(sourceId, ESyncStatusConnecting);
            iEmailIDs.Reset();

            TInt err;
            TRAP(err, 
                MailCommandL(KIMAP4MTMConnectAndSynchronise, null);
            );
            if (err != KErrNone) {
                LOG.error("Error connecting to remote email: %d", err);
                TRequestStatus* status = &iStatus;
                User::RequestComplete(status, KErrNone);
                iLastErrorCode = 1;
                iState = EDone;
            } else {
                LOG.debug("Logging on...");
                appUi->UpdateSourceStatus(sourceId, ESyncStatusLoggingOn);
                iState=EFetching;
            }
            SetActive(); //returns to active scheduler which waits for our request
        }
        break;

        case EFetching:
        {
            LOG.debug("Fetching emails...");
        	if(iStatus != KErrNone){
        	    if (iStatus == KErrImapBadLogon) {
        	        LOG.error("IMAP authentication error: %d", iStatus.Int());
        	        appUi->DisplayMessage(ERR_INVALID_CREDENTIALS, EErrorLevel);
        	    }
        	    else {
            	    // something wrong with the account!!!!
            	    LOG.error("Fetching error: %d", iStatus.Int());
        	    }
        	    iLastErrorCode = 6;
        		TRequestStatus* status = &iStatus;
                User::RequestComplete(status, KErrNone);
                iState = EDisconnecting;
        	} 
        	else {
                //Completes (with KErrNone) only when the background synchronisation
                //has finished. It is used to ensure that no UI-initiated
                //folder syncing is performed during the background synchronisation.
                TRAPD(err, MailCommandL(KIMAP4MTMWaitForBackground,null); );
                if (err != KErrNone) {
                    LOG.error("Error fetching emails: %d", err);
                    TRequestStatus* status = &iStatus;
                    User::RequestComplete(status, KErrNone);
                    iLastErrorCode = 2;
                    iState = EDone;
                } 
                else {
                    if(IMAP_RETRIEVE_BODY){
                        iState=EGetMailBody;
                    } else {
                        iState=EDisconnecting;
                    }
                }
        	}
            SetActive(); //returns to active scheduler which waits for our request
        }
        break;
        
        case EGetMailBody:
        {
            LOG.debug("Retrieving emails...");
        	TRAPD(err,
                GetMailBodyL();
            );
            if (err == KErrNone) {
                iState=EGettingMailBody;
            } else {
                iState=EDone;
                TRequestStatus* status = &iStatus;
                User::RequestComplete(status, KErrNone);
                iLastErrorCode = 3;
            }
            SetActive();
        }
        break;
        
        case EGettingMailBody:
        {
            LOG.debug("Retrieving email bodies...");
        	//Completes (with KErrNone) only when the background synchronisation
            //has finished. It is used to ensure that no UI-initiated
            //folder syncing is performed during the background synchronisation.
            TRAPD(err,
                MailCommandL(KIMAP4MTMWaitForBackground,null);
            );
            if (err == KErrNone) {
                iState=EDisconnecting;
            } else {
                iState=EDone;
                TRequestStatus* status = &iStatus;
                User::RequestComplete(status, KErrNone);
                iLastErrorCode = 4;
            }
            SetActive();
        }
        break;

        // ------------------------------------
        //    STATE: EDisconnecting
        //     Disconnects from remote mailbox
        //   note: state only for pop as
        //   it is only offline mode protocol
        // ------------------------------------
        case EDisconnecting:
        {
            LOG.debug("Disconnecting...");
            //Cancels any operations in progress
            //and sends logout messages to server.
            //As long as the background synchronisation,
            //this will also run any pending delete operations queued while offline.
            TRAPD(err,
                MailCommandL(KIMAP4MTMDisconnect,null);
            );
            if (err != KErrNone) {
            	LOG.error("Error disconnecting from email");
                TRequestStatus* status = &iStatus;
                User::RequestComplete(status, KErrNone);
                iLastErrorCode = 5;
            }

            iState=EDone;
            SetActive();
        }
        break;

        // ------------------------------------
        //    STATE: EDone
        //     Returns our machine to ready state
        // ------------------------------------
        case EDone:
        {
        	LOG.debug("Done");
        	
            TRAPD(err,
                UpdateRemoteCountL(); // updates model and informs observers
            );
            if (err != KErrNone) {
                LOG.error("Error updating remote count");
                iLastErrorCode = err;
            }
            iState = EReady;
            if(iMsvOp) {
                iMsvOp->Cancel();
            }
            delete iMsvOp;
            iMsvOp=NULL;

            // Generate event that sync is about to finish
            TCurrentSyncStatus currentSyncStatus;
            if (iLastErrorCode) {
                currentSyncStatus = ESyncStatusNotCompleted;
                LOG.error("Email sync completed with errors (%d emails received)", iTotalServerItems);
                //appUi->DisplayMessage(ERR_CONNECTING, EErrorLevel);     // TODO: better error msg
            } else {
                currentSyncStatus = ESyncStatusCompleted;
                LOG.info("Email sync completed successfully (%d emails received)", iTotalServerItems);
            }
            iSyncInProgress = EFalse;

            appUi->UpdateSourceStatus(sourceId, currentSyncStatus);

            // Save the timestamp (usually this would be done by the sync APIs)
            // At the moment we cannot save it in the sync source because times
            // are encoded as 64 bits in symbian, while the API accepts a 32
            // bits value
            ClientSettings* config = ClientSettings::getInstance();
            TTime now;
            now.UniversalTime();
            config->setLastEmailSyncTimestamp(now.Int64());
            if (iLastErrorCode == 0) {
                config->setLastEmailSyncStatus(ELastSyncSuccessful);
            } else {
                config->setLastEmailSyncStatus(ELastSyncFailed);
            }
            config->save();

            // The sync is terminated. Notify the user if necessary
            notifyUserIfNecessary();

            // This must be done as the last step, as it is going to
            // destroy this object
            syncEngine->EmailSyncEnded(currentSyncStatus);
        }
        break;

        // ------------------------------------
        //    STATE: ECanceling
        //     waits for mailcommands to cancel
        // ------------------------------------
        case ECanceling:
        {
            LOG.debug("iState = Canceling");
        	TRAPD(err,
                UpdateRemoteCountL(); // updates model and informs observers
            );
            if (err != KErrNone) {
                LOG.error("Error canceling email fetch");
            }
            iState= EReady;
            delete iMsvOp;
            iMsvOp=NULL;
            // Notify the app ui
            ClientSettings* config = ClientSettings::getInstance();
            TTime now;
            now.UniversalTime();
            config->setLastEmailSyncTimestamp(now.Int64());
            config->setLastEmailSyncStatus(ELastSyncFailed);
            appUi->UpdateSourceStatus(sourceId, ESyncStatusCanceled);
            iSyncInProgress = EFalse;
            syncEngine->EmailSyncEnded(ESyncStatusNotCompleted);
        }
        break;

        default:
            LOG.debug("iState = %d", iState);
            //EReady is default, EInitialising, EReadyButNeedsProtocol
            //handled elsewhere
            break;
    }
}

// ----------------------------------------------------------------------------
// void CImapEmailEngine::MailCommandL( const TInt aCommand,TDes8& aParams)
//  Excecutes asynchronous calls for both protocols. Note the specific
//  aSelection and aParams given to InvokeAsyncFunctionL (and later to
//  TransferCommandL) define the functioncall.
//
//  Cornerstone ii of this module.
// ----------------------------------------------------------------------------
void CImapEmailEngine::MailCommandL( const TInt aCommand, TDes8& aParams)
{
    // make sure iServiceId is a remote type service
    if( iServiceId != KMsvLocalServiceIndexEntryId ) {
        iRemoteEntries->Reset();
        iRemoteEntries->AppendL(iServiceId); //aSelection[0] is used in many mtm commands

        // get the msventry for the service id
        CMsvEntry* service = iMsvSession->GetEntryL(iServiceId);
        CleanupStack::PushL(service);

        // make sure the mtm for the service is of approriate type
        if(service->Entry().iMtm == iProtocolUid) {
            // delete and reset any outstanding operation
            if(iMsvOp) {
                iMsvOp->Cancel();
            }
            delete iMsvOp;
            iMsvOp=NULL;
            iMsvOp=iMtm->InvokeAsyncFunctionL(aCommand,*iRemoteEntries,aParams/*ptr*/,iStatus);
            if( iStatus != KRequestPending ) {
                User::Leave( iStatus.Int() ); //leave if error
            }
        }
        CleanupStack::PopAndDestroy(service);
    } else {
        User::Leave(KErrNotFound);
    }
}

void CImapEmailEngine::GetMailBodyL(void)
{
	// make sure iServiceId is a remote type service
    if( iServiceId != KMsvLocalServiceIndexEntryId ) {
        iRemoteEntries->Reset();
        iRemoteEntries->AppendL(iServiceId); //aSelection[0] is used in many mtm commands
     
        // get the msventry for the service id
        CMsvEntry* service = iMsvSession->GetEntryL(iServiceId);
        CleanupStack::PushL(service);
        CMsvEntrySelection* childrenSelection = service->ChildrenL();
        CleanupStack::PushL(childrenSelection);
        TMsvId id = (*childrenSelection)[0];
        //aSelection[1] entry id of the folder containing the messages to populate
        iRemoteEntries->AppendL(id);
        
        // please see:
        // Symbian OS v9.1  Symbian OS reference  C++ component reference 
        // Messaging MSG_EMAIL  TImImap4GetPartialMailInfo
        // and:
        // Symbian OS v9.1  Symbian OS reference  C++ component reference 
        // Messaging MSG_EMAIL  TImImap4GetMailInfo
        TImImap4GetMailInfo imap4GetMailInfo;
        imap4GetMailInfo.iDestinationFolder=iServiceId+1; //remote inbox
        //imap4GetMailInfo.iMaxEmailSize=KMaxTInt;
        imap4GetMailInfo.iMaxEmailSize=MAX_EMAIL_SIZE;
        imap4GetMailInfo.iGetMailBodyParts=EGetImap4EmailBodyText;
        TPckgBuf<TImImap4GetMailInfo> imapPackage(imap4GetMailInfo);
        
        // make sure the mtm for the service is of appropriate type
        if(service->Entry().iMtm == iProtocolUid) {
            // delete and reset any outstanding operation
            if(iMsvOp) {
                iMsvOp->Cancel();
            }
            delete iMsvOp;
            iMsvOp=NULL;
            // for a complete list of commands please see:
            // Symbian OS v9.1  Symbian OS reference  C++ component reference  Messaging EMAIL  TImap4Cmds
            iMsvOp=iMtm->InvokeAsyncFunctionL(KIMAP4MTMPopulateAllMailWhenAlreadyConnected,
                                              *iRemoteEntries,imapPackage,iStatus);

            if( iStatus != KRequestPending ) {
            	User::Leave( iStatus.Int() ); //leave if error
            }
        }
        CleanupStack::PopAndDestroy(childrenSelection);
        CleanupStack::PopAndDestroy(service);
    } else {
    	User::Leave(KErrNotFound);
    }
}

// -----------------------------------------------------------------------
// void CImapEmailEngine::HandleSessionEventL(TMsvSessionEvent aEvent,
//        TAny* /*aArg1*/,TAny* /*aArg2*/,TAny* /*aArg3*/ )
//  Messaging callback from MMsvSessionObserver
//
//  Cornerstone iii of this module.
// -----------------------------------------------------------------------
void CImapEmailEngine::HandleSessionEventL(TMsvSessionEvent aEvent, TAny* aArg1,
                                           TAny* aArg2, TAny* aArg3 )
{
    //LOG.debug("HandleSessionEventL: event %d", aEvent);

    // Most of the events are interesting only during a sync
    if (!iSyncInProgress && aEvent != MMsvSessionObserver::EMsvServerReady) {
        return;
    }

    // Process the event
    switch(aEvent)
    {
        // the messaging server is ready
        case MMsvSessionObserver::EMsvServerReady:
        {
            LOG.debug("Server ready");
            // ---------------------------------------------
            //    STATE: EInitialising
            //     Completes constructs and changes state when
            //   MsvServer session is ready
            // ---------------------------------------------
            if(iState==EInitialising) {
                iState=EReadyButNeedsProtocol;
            }
        }
        break;

        case MMsvSessionObserver::EMsvEntriesCreated:
            // MTM has just created an empty
            // message and it is about to fill it (this will generate EntriesChanged events)
            
            if (iState == EGettingMailBody) {
                // In this state, we are filling emails. (iState is always 1 position ahead! TODO: fix this!!)
                // New emails should fire a EMsvEntriesCreated event during EFetching state (not true for FP1 devices!) and
                // one or more EMsvEntriesChanged events during GetBody state. 
                // Bodies and attachments will fire EMsvEntriesCreated during GetBody state, so we need to discard them.
                // To count emails, we count the EMsvEntriesChanged events that:
                // - are fired only in GetBody state
                // - have an ID not already fired (entries can be fired more times, so we make sure they are unique checking their ID)
                //   (this applies for both EMsvEntriesChanged and EMsvEntriesCreated events)
                CMsvEntrySelection* newEntries = (CMsvEntrySelection*)aArg1;
                if (newEntries && newEntries->Count()) 
                {
                    TMsvId newEntry = newEntries->At(0);
                    //LOG.debug("New entry: id=%d, iState=%d", newEntry, iState);
                    if (iEmailIDs.Find(newEntry) == KErrNotFound) {
                        iEmailIDs.Append(newEntry);
                        // 
                    }
                }
            }
            break;

        case MMsvSessionObserver::EMsvEntriesChanged:
            // This event is generated as the MTM fills items
            
            if (iState == EGettingMailBody) {
                // In this state, we are filling emails. (iState is always 1 position ahead! TODO: fix this!!)
                // New emails should fire a EMsvEntriesCreated event during EFetching state (not true for FP1 devices!) and
                // one or more EMsvEntriesChanged events during GetBody state.
                // Bodies and attachments will fire EMsvEntriesCreated during GetBody state, so we need to discard them.
                // To count emails, we count the EMsvEntriesChanged events that:
                // - are fired only in GetBody state
                // - have an ID not already fired (entries can be fired more times, so we make sure they are unique checking their ID)
                //   (this applies for both EMsvEntriesChanged and EMsvEntriesCreated events)
                CMsvEntrySelection* newEntries = (CMsvEntrySelection*)aArg1;
                if (newEntries && newEntries->Count()) 
                {
                    TMsvId newEntry = newEntries->At(0);
                    //LOG.debug("Changed entry: id=%d, iState=%d", newEntry, iState);
                    if (iEmailIDs.Find(newEntry) == KErrNotFound) {
                        iEmailIDs.Append(newEntry);
                        LOG.debug("New Email received.");
                        appUi->UpdateSourceStatus(sourceId, ESyncStatusReceivingItemNumber, ++iTotalServerItems);
                    }
                }
            }
            break;

        case MMsvSessionObserver::EMsvEntriesDeleted:
            // Notify the App UI
            LOG.debug("Entries Deleted");
            break;

        case MMsvSessionObserver::EMsvEntriesMoved:
            //note: we could notify observers here to redraw if entry
            //would be approriate type and we would need more robust ui
            LOG.debug("Entries Moved");
            break;

        case MMsvSessionObserver::EMsvMtmGroupInstalled:
            LOG.debug("Group installed");
            break;

        case MMsvSessionObserver::EMsvMtmGroupDeInstalled:
            LOG.debug("Group deinstalled");
            break;

        case MMsvSessionObserver::EMsvGeneralError:
        case MMsvSessionObserver::EMsvServerFailedToStart:
        case MMsvSessionObserver::EMsvCorruptedIndexRebuilt:
        case MMsvSessionObserver::EMsvCloseSession:
        case MMsvSessionObserver::EMsvServerTerminated:
        case MMsvSessionObserver::EMsvMediaChanged:
        case MMsvSessionObserver::EMsvMediaUnavailable:
        case MMsvSessionObserver::EMsvMediaAvailable:
        case MMsvSessionObserver::EMsvMediaIncorrect:
        case MMsvSessionObserver::EMsvCorruptedIndexRebuilding:
            LOG.debug("IMAP error: %d", aEvent);
            appUi->UpdateSourceStatus(sourceId, ESyncStatusError, aEvent);
            iLastErrorCode = 1;
            break;
    }
}

// ---------------------------------------------
// void CImapEmailEngine::CancelOperation()
//  public interface method for user initiated
//  cancelling of email operations.
// ---------------------------------------------
//
void CImapEmailEngine::CancelOperation()
{
    LOG.debug("Cancelling IMAP synchronization");
    iState= ECanceling;    //we switch state and wait for op to cancel

    if( iMsvOp ) {
        iMsvOp->Cancel(); //iStatus now KErrCancel (-3)
    }
}

// ---------------------------------------------
// void CImapEmailEngine::DoCancel()
//  framework cancel method, no implemented here
// ---------------------------------------------
//
void CImapEmailEngine::DoCancel()
{
    // no implementation required
}

// -------------------------------------------------------
// TInt CImapEmailEngine::RunError( const TInt aError)
//  simply return error value
// -------------------------------------------------------
//
TInt CImapEmailEngine::RunError( const TInt aError)
{
	return aError;
}

// -------------------------------------
// void CImapEmailEngine::LoadMtmL()
//  common helper to load mtm context
// -------------------------------------
//
void CImapEmailEngine::LoadMtmL()
{
    if(iId == KMsvRootIndexEntryId) {
        User::Leave(KErrNotFound);
    }
    TMsvEntry tmp;
    User::LeaveIfError(iMsvSession->GetEntry(iId,iServiceId,tmp));
    iMtm = InstantiateClientMtmL(iServiceId, iProtocolUid);
}

// ------------------------------------------------------
// CBaseMtm* CImapEmailEngine::InstantiateClientMtmL(
//    TMsvId aService, const TUid aType)
//  Helper to load either imap/pop mtm using base class
// ------------------------------------------------------
//
CBaseMtm* CImapEmailEngine::InstantiateClientMtmL(TMsvId aService, const TUid aType)
{
    // create a client mtm
    CBaseMtm* newMtm = iClientReg->NewMtmL(aType);
    CleanupStack::PushL(newMtm);
    // get the entry associated with the given servive ID
    CMsvEntry* entry = iMsvSession->GetEntryL(aService);
    CleanupStack::PushL(entry);
    // set the entry as the current entry
    // mtm takes ownership of the entry
    newMtm->SetCurrentEntryL(entry);
    CleanupStack::Pop(2); //newMtm, entry

    return newMtm;
}

// -------------------------------------------------------------------
// void CImapEmailEngine::Queue()
// SetActive() or complete our own status.
// this allows the active scheduler chance to
// service other active objects before return back to our RunL
// -------------------------------------------------------------------
//
void CImapEmailEngine::Queue()
{
    if(!IsActive()) {
        SetActive();
    }

    TRequestStatus* st= &iStatus;
    User::RequestComplete(st,KErrNone);
}

TBool CImapEmailEngine::CheckIfExistsL()
{
    TMsvId id = FindServiceL();
    return ( id == KMsvRootIndexEntryId ) ? EFalse : ETrue;
}


// ---------------------------------------------
// TInt CImapEmailEngine::RemoteEmailCount()
//        get the email count as integer
// ---------------------------------------------
//
TInt CImapEmailEngine::RemoteEmailCount()
{
    return iRemoteCount;
}

// --------------------------------------------------------
// void CImapEmailEngine::RemoteFetchL()
//    enters connect state and queues own request accordingly
// --------------------------------------------------------
//
void CImapEmailEngine::RemoteFetchL()
{
    iState=EConnecting;
    iTotalServerItems = 0;
    iSyncInProgress = ETrue;
    Queue();
}

// ---------------------------------------------------------
// CMsvEntrySelection* CImapEmailEngine::UpdateEntriesL(
//        const TMsvId& aServiceId)
//  updates the message entries for viewing
// ---------------------------------------------------------
//
CMsvEntrySelection* CImapEmailEngine::UpdateEntriesL(const TMsvId& aServiceId)
{
    CMsvEntry* entry=iMsvSession->GetEntryL(aServiceId); // get the entry
    CleanupStack::PushL(entry);
    CMsvEntrySelection* entries=entry->ChildrenL(); // get the entry's children
    CleanupStack::PopAndDestroy(entry);
    return entries;
}

// -----------------------------------------------
// void CImapEmailEngine::UpdateRemoteCountL()
//  update the remote mail count
//
//    Note: only method where observers are notified
// -----------------------------------------------
//
void CImapEmailEngine::UpdateRemoteCountL()
{
    // reset the remote mail count and entries
    iRemoteCount=0;
    if (iRemoteEntries) {
        iRemoteEntries->Reset();
    }
    delete iRemoteEntries;
    iRemoteEntries=NULL;

    if(!iHasImapInbox ) {
        //leaves with KErrFound if still no folder set
        iId = FindFolderThenServiceL();
    }

    // get the current entries. Note that iId is normally a remote
    // mailbox for pop and first folder( inbox ) for imap. which is
    // why we have to check if folders are already sync if protocol
    // is imap, so we wont draw the screen with wrong parent and output
    // folders to the listbox and not the message entries we want to.
    iRemoteEntries=UpdateEntriesL(iId);

    // if the nunber has changed then inform the observers
    if(iRemoteEntries->Count()!=iRemoteCount) {
        iRemoteCount=iRemoteEntries->Count();
    }
}

TBool CImapEmailEngine::IsEngineReady()
{
    return (iClientReg!=NULL)?(ETrue):(EFalse);
}

void CImapEmailEngine::PrepareL()
{
    // We wait for the engine to get ready
    iProtocolUid = KUidMsgTypeIMAP4;
    iState = EReady;
    iId = FindServiceL();
    LoadMtmL();
    TRAPD(err, iId = FindFolderThenServiceL());
    if (err != KErrNone) {
        goto finally;
    }
    UpdateRemoteCountL(); //we draw the message entries if any

    // set current entry to IMAP Inbox and set the observer
    // we use it to monitor mail message addition through
    // HandleEntryEventL()
finally:
    LOG.debug("Starting email observer");
    iEntry = iMsvSession->GetEntryL(iId);
    iEntry->AddObserverL(*this);
}



// --------------------------------------------------------------
// TMsvId CImapEmailEngine::FindServiceL(TUid aType)
//  find the message id of a service given a UID
//  this function will return the first service id for given type
// --------------------------------------------------------------
//
TMsvId CImapEmailEngine::FindServiceL()
{
    HBufC* name = charToNewBuf(SYNC_PROFILE_NAME);
    CleanupStack::PushL(name);

    // select the root index to start the search
    CMsvEntry* currentEntry = iMsvSession->GetEntryL(KMsvRootIndexEntryId);
    CleanupStack::PushL(currentEntry);

    // don't sort the entries
    currentEntry->SetSortTypeL(TMsvSelectionOrdering(KMsvNoGrouping,EMsvSortByNone, ETrue));

    TMsvId rc = KMsvRootIndexEntryId;
    TInt count=currentEntry->Count();
    // loop for every child entry of the root index
    for(TInt i = 0;i<count;i++) {
        const TMsvEntry& child = (*currentEntry)[i];

        if (child.iMtm == KUidMsgTypeIMAP4 && child.iDetails.Compare(*name) == 0) {
            rc = child.Id();
            break;
        }
    }

    CleanupStack::PopAndDestroy(currentEntry);
    CleanupStack::PopAndDestroy(name);
    // return the service id of the type if found.
    // otherwise return the root index
    return rc;
}

// ---------------------------------------------------------------
// TMsvId CImapEmailEngine::FindFolderThenServiceL(TUid aType)
//  this function will return the first folder of first
//  service of given type or if it doesnt exist then
//    this function will leave with KErrNotFound as most of
//    the usage sitations couldnt handle mailbox instead of
//  folder.
// ----------------------------------------------------------------
//
TMsvId CImapEmailEngine::FindFolderThenServiceL()
{
    HBufC* name = charToNewBuf(SYNC_PROFILE_NAME);
    CleanupStack::PushL(name);

    // select the root index to start the search
    CMsvEntry* currentEntry = iMsvSession->GetEntryL(KMsvRootIndexEntryId);
    CleanupStack::PushL(currentEntry);

    // don't sort the entries
    currentEntry->SetSortTypeL(TMsvSelectionOrdering(KMsvNoGrouping,EMsvSortByNone, ETrue));

    TMsvId rc = KMsvRootIndexEntryId;
    TInt count=currentEntry->Count();
    // loop for every child entry of the root index
    for(TInt i = 0;i<count;i++) {
        const TMsvEntry& child = (*currentEntry)[i];

        // is the current child the same type as the type we are looking for ?
        if (child.iMtm == KUidMsgTypeIMAP4 && child.iDetails.Compare(*name) == 0) {

            // selects first entry as index entry
            CMsvEntry* firstEntry = iMsvSession->GetEntryL(child.Id());
            CleanupStack::PushL(firstEntry);
            TInt innercount=firstEntry->Count();

            if( innercount ) {
                //if has childs == folders take first
                const TMsvEntry& folder = (*firstEntry)[0];
                rc=folder.Id();
                iHasImapInbox=ETrue;
            } else {
                iHasImapInbox=EFalse;
            }
            CleanupStack::PopAndDestroy(firstEntry);
            break;
        }
    }//for

    CleanupStack::PopAndDestroy(currentEntry);
    CleanupStack::PopAndDestroy(name);

    if( !iHasImapInbox ) {
        User::Leave( KErrNotFound );
    }

    // return the service id of the type if found.
    return rc;
}



bool CImapEmailEngine::CheckSyncInProgress() {
    return iSyncInProgress;
}

// Get an instance of the ECOM interface to the notification system.
TBool CImapEmailEngine::notifyUserIfNecessary()
{
    TInt err;
    MNcnNotification* notifyNewMessageEcom = 0;
    TRAP(err, notifyNewMessageEcom = MNcnNotification::CreateMNcnNotificationL());

    // Check for errors
    if(notifyNewMessageEcom && err == KErrNone) {
        // The interface is supported on this phone.
        CleanupDeletePushL(notifyNewMessageEcom);

        // Now notify the framework that new messages have been received
        // First create a descriptor array for aInfo - even though it isn't used
        // presently.
        CDesCArray* tempArray = new (ELeave) CDesCArrayFlat(3);
        CleanupStack::PushL(tempArray);

        // Notify the framework that there are new messages.
        // This can return an error value.
        err = notifyNewMessageEcom->NewMessages(iServiceId,
                                                MNcnNotification::EIndicationNormal,
                                                *tempArray);
        if (err != KErrNone) {
            LOG.error("User notification failed (%d)", err);
        }

        // Cleanup the descriptor array.
        CleanupStack::PopAndDestroy(tempArray);

        // Clean up the ECOM interface now.
        CleanupStack::PopAndDestroy(notifyNewMessageEcom);
    } else {
        LOG.error("Notification plugin is not available");
    }
    // Return true on success
    return err == KErrNone;
}

//This is the callback for CMsvEntry changes; since we can monitor child addition, we use it 
//to monitor IMAP Inbox for every message addition
void CImapEmailEngine::HandleEntryEventL(TMsvEntryEvent aEvent, TAny *aArg1, TAny *aArg2, TAny *aArg3)
{
    //LOG.debug("CImapEmailEngine::HandleEntryEventL (event %d)", aEvent);
	switch (aEvent)
	{
		case EMsvNewChildren:
		{
			// A new message has been added to the Inbox
			//appUi->UpdateSourceStatus(sourceId, ESyncStatusReceivingItemNumber, ++iTotalServerItems);
			//LOG.debug("old2 detected new");
			break;
		}
		
		case EMsvEntryChanged:
	    {
	        // A new message has been added to the Inbox
	        //appUi->UpdateSourceStatus(sourceId, ESyncStatusReceivingItemNumber, ++iTotalServerItems);
	        //LOG.debug("old2 detected change");
	        break;
	    }
		case EMsvChildrenChanged:
		case EMsvDeletedChildren:
		case EMsvEntryDeleted:
		case EMsvContextInvalid:
		case EMsvChildrenMissing:
		case EMsvChildrenInvalid:
		case EMsvEntryMoved:
			break;
	}
}
