/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <akntitle.h> 
#include <barsread.h>
#include <npdapi.h>         // CNotepadApi (View Log)
#include "uid3.h"

#include "base/SymbianLog.h"
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "base/util/symbianUtils.h"

#include "commonDefs.h"
#include "customization.h"
#include "Funambol.hrh"
#include "LoggingDialog.h"
#include "FunambolAppUi.h"
#include "ClientSettings.h"
#include "utils.h"

using namespace Funambol;


CLoggingDialog* CLoggingDialog::NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI)
{
    CLoggingDialog* self = new (ELeave)CLoggingDialog();
    CleanupStack::PushL(self);
    self->ConstructL(aMenuTitleResourceId, aAppUI);
    CleanupStack::Pop(self);
    return self;
}

void CLoggingDialog::ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI)
{
    iAppUI = aAppUI;
    iSettingList = NULL;
    CAknDialog::ConstructL(aMenuTitleResourceId);
}

CLoggingDialog::CLoggingDialog() : iLogLevel(LOG_LEVEL_INFO) {}


CLoggingDialog::~CLoggingDialog()
{
    if (iSettingList) {
        delete iSettingList;
        iSettingList = NULL;
    }
}

void CLoggingDialog::PreLayoutDynInitL()
{
    if (!iSettingList) // if we haven't already created iSettingList
    {
        // construct control and set parent
        iSettingList = new (ELeave) CCommonOptionsSettingList();
        iSettingList->SetMopParent(this);

        // Loads data into the setting list.
        load();

        // construct control based on resource definition
        TResourceReader reader;

        // CreateResourceReaderLC will allocate a buffer to be used by
        // the TResourceReader. This buffer is pushed onto the cleanup
        // stack - not the TResourceReader itself
        iCoeEnv->CreateResourceReaderLC(reader, R_LOGGING_SETTING_LIST);
        iSettingList->ConstructFromResourceL(reader);

        // Clean up the buffer allocated above, NOT the reader itself.
        // Cannot use expected item overload of PopAndDestroy() as buffer 
        // is inaccessible.
        CleanupStack::PopAndDestroy();
        
        // Removes the vertical scrollbar
        iSettingList->initScrollbar();

        // Activate the settings list
        iSettingList->ActivateL();
    }
    
    iAppUI->changeAppTitle(R_STRING_LABEL_LOGGING);
    CAknDialog::PreLayoutDynInitL();
}



TBool CLoggingDialog::OkToExitL(TInt aButtonId)
{
    //LOG.debug("CLoggingDialog::OkToExitL: button %d", aButtonId);
    
    // Returning ETrue causes the form to exit
    switch (aButtonId)
    {
        case EAknSoftkeyBack:
        {
            // Save data into the config: will start/stop services if necessary...
            save();
            return ETrue;
        }
        case ESubOptionsChange:      
        {
            if (MenuShowing()) { HideMenu(); }
            iSettingList->EditCurrentItemL();
            break;
        }
        // Not used...
        /*case EEikCmdExit:
        case EAknSoftkeyExit:
        {
            iAppUI->Exit();
            break;
        }*/
        default:
        {
            // Other commands forwarded to CAknDialog.
            return CAknDialog::OkToExitL(aButtonId);  
        }
    }
    return EFalse;
}


TKeyResponse CLoggingDialog::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType != EEventKey) {
        return EKeyWasNotConsumed;
    }
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3)          // ???????? why this code ????????
    {
        iSettingList->EditCurrentItemL();
        return EKeyWasConsumed;
    }
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to setting list
        case EKeyDownArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}


void CLoggingDialog::ProcessCommandL( TInt aCommand )
{
    //LOG.debug("CLoggingDialog::ProcessCommandL - event received: %d", aCommand);
    switch( aCommand )
    {
        case ESubOptionsChange:      
        {
            if (MenuShowing()) { HideMenu(); }
            iSettingList->EditCurrentItemL();
            break;
        }
        case EAknSoftkeyCancel:
        case EAknSoftkeyExit:
        {
            // Forward the command to OkToExitL, so that we can
            // exit this dialog.
            TryExitL(aCommand);     // will call OkToExitL()
            break;
        }
        
        case EViewLog:
        {
            // Open log with Notepad
            openLogFile();
        }

        default: 
        {
            // Other commands forwarded to CAknDialog.
            CAknDialog::ProcessCommandL(aCommand);
        }
    }
}



void CLoggingDialog::load()
{
    LOG.info("Reading Logging data...");
    ClientSettings* config = ClientSettings::getInstance();
    config->open();

    // Read values from config and set values into the settinglist
    iLogLevel = config->getDeviceConfig().getLogLevel();
    iSettingList->setLogLevel(iLogLevel);
    
    config->close();
}


TBool CLoggingDialog::save()
{
    // Read values inserted by the user
    LogLevel newLogLevel = (LogLevel)iSettingList->getLogLevel();

    // Check if values changed
    if (newLogLevel == iLogLevel) {
        LOG.debug("Data unchanged -> no need to save config.");
        return EFalse;
    }
    
    LOG.info("Saving Log level = %d", newLogLevel);
    ClientSettings* config = ClientSettings::getInstance();
    config->open();
    
    
    // ---- Log level ----
    if (newLogLevel != iLogLevel) {
        config->getDeviceConfig().setLogLevel(newLogLevel);
        LOG.setLevel(newLogLevel);
    }


    // Persist config to disk.
    config->save();
    config->close();
    
    return ETrue;
}

void CLoggingDialog::openLogFile()
{
    TInt err = KErrNone;
    LOG.info("Opening log file with notepad");
    
    // Get log name
    RBuf name;
    StringBuffer logName(LOG_FILE_PATH);
    logName += LOG_FILE_NAME;
    name.Assign(stringBufferToNewBuf(logName));
    
    RFs fsSession;
    RFile file;
    fsSession.Connect();
    fsSession.ShareAuto();
    err = file.Open(fsSession, name, EFileRead|EFileShareAny);
    

    /* --- Why not working??? ---
    TRAP( err, CNotepadApi::ExecFileViewerL(name); )
    LOG.debug("*** err = %d", err);
    file.Close();
    fsSession.Close();
    name.Close();*/
    
    TEntry entry;
    fsSession.Entry(name, entry);
    TInt size = entry.iSize;
    name.Close();
    
    // Get log content (utf8)
    RBuf8 content8;
    content8.Create(size);
    file.Read(content8);
    file.Close();
    fsSession.Close();
    
    // Convert it to unicode
    RBuf content16;
    content16.Create(size);
    HBufC* buf = CnvUtfConverter::ConvertToUnicodeFromUtf8L(content8);
    content8.Close();      // Release mem as soon as we can!
    content16.Assign(buf);
    
    HBufC* title = iEikonEnv->AllocReadResourceLC(R_STRING_VIEWLOG_TITLE);
    
    //
    // Open notepad with this content, in read-only mode.
    //
    TRAP( err, CNotepadApi::ExecReadOnlyTextViewerL(content16, *title); )
    if (err) {
        LOG.error("Error executing TextViewer: code %d", err);
    }
    
    content16.Close();              // Release mem as soon as we can!
    CleanupStack::PopAndDestroy();  // title
}
