/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include <e32std.h>
#include <eikenv.h>
#include <coeutils.h>
#include <s32file.h>

#include "Model.h"
#include "FunambolDocument.h"

CModel::CModel( CFunambolDocument* aDocument ) : iDocument( aDocument )    
{
}

void CModel::ConstructL()
{
}

CModel::~CModel()
{
}

CModel* CModel::NewL( CFunambolDocument* aDocument )
{
    CModel* self = new (ELeave) CModel( aDocument );
    
    CleanupStack::PushL( self );
    self->ConstructL();
    CleanupStack::Pop();

    return self;
}


void CModel::Reset()
{
    iAutoStartEnable= ETrue;
}

void CModel::ExternalizeL() const
{
    TFileName fileName;

    TParse parse;
    parse.SetNoWild( iDocument->Application()->AppFullName(), NULL, NULL );

    TBuf<50> privatePath;
    CEikonEnv::Static()->FsSession().PrivatePath( privatePath );

    #if defined(__WINS__) || defined (_WINSCW__)
        _LIT( KDriveLetter, "C:" );
        fileName.Copy( KDriveLetter );
        fileName.Append( privatePath );
    #else
        fileName.Copy( parse.Drive() );
        fileName.Append( privatePath );    
    #endif

    fileName.Append( KSettingFileName );

    RFs fileSession;
    User::LeaveIfError( fileSession.Connect() );
    CleanupClosePushL( fileSession );

    RFileWriteStream outputFileStream;
    if ( KErrNone != outputFileStream.Replace( fileSession,
                                 fileName, EFileShareExclusive) )
    {
        // Cleanup the file session
        CleanupStack::PopAndDestroy( ); // fileSession
        return;
    }

    CleanupClosePushL( outputFileStream );
    outputFileStream.WriteInt8L( iAutoStartEnable );
 
    CleanupStack::PopAndDestroy( );  // outputFileStream
    CleanupStack::PopAndDestroy( );  // fileSession

}

void CModel::InternalizeL()
{
    TFileName fileName;

    TParse parse;
    parse.SetNoWild( iDocument->Application()->AppFullName(), NULL, NULL );

    TBuf<50> privatePath;
    CEikonEnv::Static()->FsSession().PrivatePath( privatePath );

    #if defined(__WINS__) || defined (_WINSCW__)
        _LIT( KDriveLetter, "C:" );
        fileName.Copy( KDriveLetter );
        fileName.Append( privatePath );
    #else
        fileName.Copy( parse.Drive() );
        fileName.Append( privatePath );    
    #endif

    fileName.Append( KSettingFileName );

    if ( !ConeUtils::FileExists( fileName ) ) {
        ConeUtils::EnsurePathExistsL( fileName );
        
        // We need to create the file name here
        RFileWriteStream fileWStream;
        CleanupClosePushL( fileWStream );
        User::LeaveIfError( fileWStream.Create( CEikonEnv::Static()->FsSession(),
                                    fileName, EFileShareExclusive) );
        
        fileWStream.CommitL();
        CleanupStack::PopAndDestroy();  // fileWStream
        
        Reset();

        // If the file does not exist, then create it!!!
        ExternalizeL();
    }

    RFs fileSession;
    User::LeaveIfError( fileSession.Connect() );
    CleanupClosePushL( fileSession );

    RFileReadStream inputFileStream;
    User::LeaveIfError( inputFileStream.Open( fileSession,
                                fileName, EFileShareReadersOnly ) );
    CleanupClosePushL( inputFileStream );
    
    // Read autostart flag
    iAutoStartEnable = inputFileStream.ReadInt8L();

    CleanupStack::PopAndDestroy( );  // inputFileStream
    CleanupStack::PopAndDestroy( );  // fileSession
}

