/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <akntitle.h>
#include <barsread.h>
#include "uid3.h"

#include "base/SymbianLog.h"
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "base/util/symbianUtils.h"

#include "commonDefs.h"
#include "Funambol.hrh"
#include "PIMOptionsDialog.h"
#include "FunambolAppUi.h"
#include "ClientSettings.h"
#include "utils.h"

using namespace Funambol;


CPIMOptionsDialog* CPIMOptionsDialog::NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI, const TInt aSourceID)
{
    LOG.debug("newL");
    CPIMOptionsDialog* self = new (ELeave)CPIMOptionsDialog();
    CleanupStack::PushL(self);
    self->ConstructL(aMenuTitleResourceId, aAppUI, aSourceID);
    CleanupStack::Pop(self);
    return self;
}

void CPIMOptionsDialog::ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI, const TInt aSourceID)
{
    iAppUI       = aAppUI;
    iSettingList = NULL;
    iSourceID    = aSourceID;
    iSourceName  = getSourceNameFromID(iSourceID);
    
    CAknDialog::ConstructL(aMenuTitleResourceId);
}

CPIMOptionsDialog::CPIMOptionsDialog() : iSourceName(""), 
                                         iSyncEnabled(ETrue), 
                                         iSyncMode(ESyncModeNormal)
{
}


CPIMOptionsDialog::~CPIMOptionsDialog()
{
    if (iSettingList) {
        delete iSettingList;
        iSettingList = NULL;
    }
}

void CPIMOptionsDialog::PreLayoutDynInitL()
{
    if (!iSettingList) // if we haven't already created iSettingList
    {
        // construct control and set parent
        iSettingList = new (ELeave) CCommonOptionsSettingList();
        iSettingList->SetMopParent(this);

        // Loads data into the setting list.
        load();

        // construct control based on resource definition
        TResourceReader reader;

        // CreateResourceReaderLC will allocate a buffer to be used by
        // the TResourceReader. This buffer is pushed onto the cleanup
        // stack - not the TResourceReader itself
        iCoeEnv->CreateResourceReaderLC(reader, R_PIM_SETTING_LIST);
        iSettingList->ConstructFromResourceL(reader);

        // Clean up the buffer allocated above, NOT the reader itself.
        // Cannot use expected item overload of PopAndDestroy() as buffer 
        // is inaccessible. 
        CleanupStack::PopAndDestroy();
        
        // Removes the vertical scrollbar
        iSettingList->initScrollbar();

        // Activate the settings list
        iSettingList->ActivateL();
    }
    
    // Set the correct title. TODO: move to controller?
    if (iSourceID == SOURCE_CONTACTS) {
        iAppUI->changeAppTitle(R_STRING_CONTACTS_OPTIONS_TITLE);
    } else if (iSourceID == SOURCE_CALENDAR){
        iAppUI->changeAppTitle(R_STRING_CALENDAR_OPTIONS_TITLE);
    } else if (iSourceID == SOURCE_NOTES){
        iAppUI->changeAppTitle(R_STRING_NOTES_OPTIONS_TITLE);
    }
    
    CAknDialog::PreLayoutDynInitL();
}



TBool CPIMOptionsDialog::OkToExitL(TInt aButtonId)
{
    //LOG.debug("CPIMOptionsDialog::OkToExitL: button %d", aButtonId);
    
    // Returning ETrue causes the form to exit
    switch (aButtonId)
    {
        case EAknSoftkeyBack:
        {
            // Save data into the config.
            save();
            return ETrue;
        }
        case ESubOptionsChange:      
        {
            //if (MenuShowing()) { HideMenu(); }    not using menu
            iSettingList->EditCurrentItemL();
            break;
        }
        // Not used...
        /*case EEikCmdExit:
        case EAknSoftkeyExit:
        {
            iAppUI->Exit();
            break;
        }*/
        default:
        {
            // Other commands forwarded to CAknDialog.
            return CAknDialog::OkToExitL(aButtonId);  
        }
    }
    return EFalse;
}


TKeyResponse CPIMOptionsDialog::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if (aType != EEventKey) {
        return EKeyWasNotConsumed;
    }
    
    if (aKeyEvent.iScanCode == EStdKeyDevice3)          // ???????? why this code ????????
    {
        iSettingList->EditCurrentItemL();
        return EKeyWasConsumed;
    }
    
    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to setting list
        case EKeyDownArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iSettingList->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}



void CPIMOptionsDialog::load()
{
    LOG.info("Reading %s options data...", iSourceName.c_str());
    ClientSettings* config = ClientSettings::getInstance();
    config->open();

    // Read values from config
    SyncSourceConfig* ssConfig = config->getSyncSourceConfig(iSourceName.c_str());
    if (!ssConfig) {
        LOG.error("Config not found for source '%s'", iSourceName.c_str());
        return;
    }
    
    iSyncEnabled = ssConfig->isEnabled();
    
    // Show the sync direction even if source is disabled
    StringBuffer syncType = ssConfig->getSync();
    if (syncType == SYNC_TWO_WAY_TEXT) {
        iSyncMode = ESyncModeNormal;
    }
    else if (syncType == SYNC_ONE_WAY_FROM_CLIENT_TEXT) {
        iSyncMode = ESyncModeToServerOnly;
    }
    else {
        iSyncMode = ESyncModeToPhoneOnly;
    }
    
    // Set values into the settinglist
    iSettingList->setSyncEnabled(iSyncEnabled);
    iSettingList->setSyncMode(iSyncMode);
    
    config->close();
}

TBool CPIMOptionsDialog::save()
{
    // Read values inserted by the user
    TBool newSyncEnabled = iSettingList->getSyncEnabled();
    TInt newSyncMode     = iSettingList->getSyncMode();

    // Check if values changed
    if ((newSyncEnabled == iSyncEnabled) && (newSyncMode == iSyncMode)) {
        LOG.debug("Data unchanged -> no need to save config.");
        return EFalse;
    }
    
    // Store values to config
    LOG.info("Saving %s options data...", iSourceName.c_str());
    ClientSettings* config = ClientSettings::getInstance();
    config->open();
    SyncSourceConfig* ssConfig = config->getSyncSourceConfig(iSourceName.c_str());
    if (!ssConfig) {
        LOG.error("Config not found for source '%s'", iSourceName.c_str());
        return EFalse;
    }
    
    ssConfig->setIsEnabled(newSyncEnabled);

    if (newSyncMode == ESyncModeNormal) {
        ssConfig->setSync(SYNC_TWO_WAY_TEXT);
    }
    else if (newSyncMode == ESyncModeToServerOnly) {
        ssConfig->setSync(SYNC_ONE_WAY_FROM_CLIENT_TEXT);
    }
    else {
        ssConfig->setSync(SYNC_ONE_WAY_FROM_SERVER_TEXT); 
    }
    
    // The scheduler could be disabled if no sources are active.
    if (config->getSyncUsing() == ESyncUsingScheduled) {
        iAppUI->StartServices();
    }


    config->save();
    config->close();
    
    return ETrue;
}
