/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <aknprogressdialog.h>
#include <eikprogi.h>
#include <aknenv.h>

#include "base/Log.h"

#include "commonDefs.h"
#include "Funambol.hrh"
#include "ClientSettings.h"
#include "FunambolAppUi.h"
#include "uid3.h"

#include "ProgressDialog.h"

using namespace Funambol;

ProgressDialog::ProgressDialog(bool cancelButton) : iProgressDialog(NULL),
                                                    iProgressInfo(NULL),
                                                    iWaitDialog(NULL),
                                                    iCancelButton(cancelButton)
{
}

ProgressDialog::~ProgressDialog()
{
    if(iProgressDialog) {
        iProgressDialog->ProcessFinishedL(); 
        delete iProgressDialog;
    }
 
    if(iWaitDialog) {
        iWaitDialog->ProcessFinishedL(); 
        delete iWaitDialog;
    }
}
 
 
void ProgressDialog::StartWaitNoteL()
{
    iWaitDialog = new (ELeave) CAknWaitDialog(
            (REINTERPRET_CAST(CEikDialog**, &iWaitDialog)), 
            ETrue);
    if (iCancelButton) {
        iWaitDialog->PrepareLC(R_WAIT_NOTE_SOFTKEY_CANCEL);
    } else {
        iWaitDialog->PrepareLC(R_WAIT_NOTE_SOFTKEY_NONE);
    }
    iWaitDialog->SetCallback(this);
    iWaitDialog->RunLD();
}
 
void ProgressDialog::StartProgressNoteL(TInt aFinalValue)
{
    iProgressDialog = new (ELeave) CAknProgressDialog(
             (REINTERPRET_CAST(CEikDialog**, &iProgressDialog)),
             ETrue);
    iProgressDialog->PrepareLC(R_PROGRESS_NOTE);
    iProgressInfo = iProgressDialog->GetProgressInfoL();
    iProgressDialog->SetCallback(this);
    iProgressDialog->RunLD();
    iProgressInfo->SetFinalValue(aFinalValue);
}
 
void ProgressDialog::UpdateProcessL(TInt aProgress,
                                    const TDesC& aProgressText)
{
    if (iWaitDialog) {
        iWaitDialog->SetTextL(aProgressText);
    }

    if(iProgressDialog) {
        iProgressDialog->SetTextL(aProgressText);
    }

    if(iProgressInfo) {
        iProgressInfo->SetAndDraw(aProgress);
    }
}
 
void ProgressDialog::DialogDismissedL(TInt aButtonId)
{
   iProgressDialog = NULL;
   iProgressInfo = NULL;
   iWaitDialog = NULL;

   if (aButtonId == EAknSoftkeyCancel) {
      // cancel any process in here
   }
}

