/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <apgcli.h>  
#include <apacmdln.h> 

#include "base/SymbianLog.h"
#include "syncml/core/SyncNotification.h" 
#include "base/util/ArrayList.h"
#include "push/CTPService.h"

#include "PushEngine.h"
#include "commonDefs.h"
#include "customization.h"
#include "utils.h"

#include "SMSPushSubscriber.h"
#include "SMSPushProperty.h"

#include "SMSPushEngine.h"

using namespace Funambol;

CSMSPushEngine* CSMSPushEngine::NewL(CPushEngine* aPushEngine)
{
    CSMSPushEngine* self = CSMSPushEngine::NewLC(aPushEngine);
    CleanupStack::Pop( self );
    return self;
}

CSMSPushEngine* CSMSPushEngine::NewLC(CPushEngine* aPushEngine)
{
    CSMSPushEngine* self = new (ELeave) CSMSPushEngine(aPushEngine);
    CleanupStack::PushL( self );
    self->ConstructL();
    return self;
}

CSMSPushEngine::CSMSPushEngine(CPushEngine* aPushEngine) : 
    iPushEngine(aPushEngine), iRunning(false) {}

CSMSPushEngine::~CSMSPushEngine() 
{
    if (iTimer) {
        iTimer->Cancel();
        delete iTimer;
    }

    delete iPushSubscriber;
}

// setup periodic timer for push server watchdog
void CSMSPushEngine::ConstructL()
{
    iTimer = CPeriodic::NewL( CActive::EPriorityStandard );

    // start push server watchdog (it will start push server if it's not running)
    iTimer->Start(0, SMS_PUSH_SERVER_CHECK_FREQ, TCallBack(CSMSPushEngine::PushServerWatchDog, this));
}

void CSMSPushEngine::start()
{
    if (iRunning == false) {
        LOG.info("Starting SMS push engine");

        // creating subscriber starts automatically watching 
        // for server published properties
        if (iPushSubscriber == NULL) {
            LOG.info("starting SMS push subscriber");
            iPushSubscriber = CSMSPushSubscriber::NewL(this);
        }

        iRunning = true;
    } else {
        LOG.error("SMSPushEngine: start command received when already running");
    }
}

void CSMSPushEngine::stop(TBool aWatchDogStopFlag)
{
    LOG.info("Stopping SMS push listener");

    if (aWatchDogStopFlag) {
        // stop push server watchdog
        if (iTimer) {
            iTimer->Cancel();
        }
    }
 
    delete iPushSubscriber;
    iPushSubscriber = NULL;

    iRunning = false;
}

TInt CSMSPushEngine::PushServerWatchDog(TAny *aPtr)
{
    CSMSPushEngine* self = (CSMSPushEngine*)aPtr;

    LOG.debug("PushServerWatchDog: checking for push server process");

    if (self->CheckPushServerProcess() == EFalse) {
        LOG.info("push server not running: starting it");
        self->StartPushServerProcessL();
    }

    return KErrNone;
}

TBool CSMSPushEngine::CheckPushServerProcess()
{
    TBool ret = EFalse;
    TUid pushServerUid = { SMS_PUSH_SERVER_SID };
    // check for push server running
    ret = CheckProcessRunning( pushServerUid );
    
    return ret;
}

void CSMSPushEngine::StartPushServerProcessL()
{
    TPtrC pushServerName( (const TText*) SMS_PUSH_SERVER_APPNAME );
    
    StartServerProcessL( pushServerName );
}

void CSMSPushEngine::onSMSPushReceived(RArray<TInt>& aSourceList)
{
    if (aSourceList.Count() > 0) {
        iPushEngine->onNotificationReceived(aSourceList);    
    }
}

