/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/SymbianLog.h"
#include "commonDefs.h"
#include "customization.h"

#include "SMSPushEngine.h"
#include "SMSPushProperty.h"
#include "SMSPushSubscriber.h"

using namespace Funambol;

CSMSPushSubscriber* CSMSPushSubscriber::NewL(CSMSPushEngine* aPushEngine)
{
    CSMSPushSubscriber* self = CSMSPushSubscriber::NewLC(aPushEngine);
    CleanupStack::Pop( self );
    return self;
}

CSMSPushSubscriber* CSMSPushSubscriber::NewLC(CSMSPushEngine* aPushEngine)
{
    CSMSPushSubscriber* self = new (ELeave) CSMSPushSubscriber(aPushEngine);
    CleanupStack::PushL( self );
    self->ConstructL();

    return self;
}

CSMSPushSubscriber::CSMSPushSubscriber(CSMSPushEngine* aPushEngine) : 
    CActive(EPriorityStandard), iPropertyUid(KSMSPushPropUid), iPropertyKey(EPushReceived), iSmsPushEngine(aPushEngine) {}
   

CSMSPushSubscriber::~CSMSPushSubscriber() 
{
    Cancel();
    iProperty.Close();
}

void CSMSPushSubscriber::ConstructL()
{
    CActiveScheduler::Add(this); 
    User::LeaveIfError(iProperty.Attach(iPropertyUid, iPropertyKey));
    RunL();
}

TInt CSMSPushSubscriber::RunError(TInt aError) 
{
    LOG.error("RunL() error: %d", aError);

    return KErrNone;
}

void CSMSPushSubscriber::DoCancel()
{
    iProperty.Cancel();
}

void CSMSPushSubscriber::RunL()
{
    TInt val;
    RArray<TInt> sourceList;
     
    if ( iProperty.Get( val ) == KErrNotFound ) {
        LOG.error("property deleted!");
    } else {
        if (val) {
            LOG.debug("sync notification received");

            if (val & EMailPush) {
                sourceList.Append(SOURCE_MAIL);
            } 

            if (val & EContactsPush) {
                sourceList.Append(SOURCE_CONTACTS);
            }

            if (val & ECalendarPush) {
                sourceList.Append(SOURCE_CALENDAR);
            }

            if (val & ENotesPush) {
                sourceList.Append(SOURCE_NOTES);
            }

            LOG.debug("reset property valued after notification");
            iProperty.Set(ENullPush);
        }
    }

    if (sourceList.Count() > 0) {
        iSmsPushEngine->onSMSPushReceived(sourceList);
    }

    // property subscrption postponed because we update value
    iProperty.Subscribe( iStatus );
    SetActive();
}
