/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <eikedwin.h>
#include <eikseced.h>
#include <avkon.rsg>
#include <eikmenup.h>
#include <barsread.h>
#include "uid3.h"

#include "base/Log.h"
#include "base/util/stringUtils.h"
#include "push/FThread.h"

#include "commonDefs.h"
#include "customization.h"
#include "Funambol.hrh"
#include "ClientSettings.h"
#include "FunambolAppUi.h"
#include "ProgressDialog.h"

#include "UpgradeDialog.h"

using namespace Funambol;


CUpgradeDialog* CUpgradeDialog::NewL(CFunambolAppUi* appUi,
                                     TBool           aRecommended,
                                     TInt&           aChoice)
{
    CUpgradeDialog* self = new (ELeave)CUpgradeDialog(appUi, aRecommended, aChoice);
    CleanupStack::PushL(self);
    if (aRecommended) {
        self->ConstructL(R_MENUBAR_SUB_OPTIONS);
    } else {
        self->ConstructL(R_UPGRADE_MENUBAR);
    }
    CleanupStack::Pop(self);
    return self;
}

CUpgradeDialog::CUpgradeDialog(CFunambolAppUi* appUi, TBool aRecommended, TInt& aChoice)
                                          : iAppUI(appUi),
                                            iRecommended(aRecommended),
                                            iChoice(aChoice)
{
}


CUpgradeDialog::~CUpgradeDialog()
{
}

void CUpgradeDialog::ShowLD()
{
    // Force the client to go to foreground.
    // This is necessary to make sure the user see this popup, since he has
    // to do a choice (see bug 9296).
    if (iAppUI) {
        iAppUI->GoToForeground();
    }
    
    if (iRecommended) {
        this->ExecuteLD(R_RECOMMENDED_UPGRADE_DIALOG);
    } else {
        this->ExecuteLD(R_UPGRADE_DIALOG);
    }
}

void CUpgradeDialog::ProcessCommandL( TInt aCommand )
{
    TryExitL(aCommand);     // will call OkToExitL()
}

void CUpgradeDialog::SetSizeAndPosition( const TSize& Size )
{
    CAknDialog::SetSizeAndPosition( Size );
        
    TRect cRect = ((CAknAppUi*)CCoeEnv::Static()->AppUi())->ClientRect();
    TRect rect = Rect();

    SetExtent(TPoint(Position().iX, cRect.iBr.iY - rect.Height()), rect.Size());
}

TBool CUpgradeDialog::OkToExitL(TInt aButtonId)
{
    switch (aButtonId) {
        case ENow:
        case ELater:
        case ESkip:
            iChoice = aButtonId;
            return ETrue;
        default:
            // This must be the option button in non Recommended update
            return CAknDialog::OkToExitL(aButtonId);
    }
}

