/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/util/stringUtils.h"
#include "commonDefs.h"
#include "base/globalsdef.h"
#include "base/SymbianLog.h"
#include "base/util/symbianUtils.h"
#include "base/util/StringBuffer.h"
#include "base/base64.h"
#include "FunambolAppUi.h"
#include "ClientSettings.h"
#include "base/util/XMLProcessor.h"
#include "XmlRpcEngine.h"

// XML-RPC tags
_LIT(KMethodCallTag, "methodCall");
_LIT(KMethodNameTag, "methodName");


xmlRpcHandler CXmlRpcEngine::xmlRpcHandlersTable [] =  {
    { "GetUserAccount", CXmlRpcEngine::GetUserAccountCallBack },
    {  NULL, NULL }
};


const char* CXmlRpcEngine::xmlRpcResponseFmt = 
    "<?xml version=\"1.0\"?>    \
        <methodResponse>        \
            <params>            \
                %s              \
            </params>           \
        </methodResponse>";

const char* CXmlRpcEngine::xmlRcpResponseStringParamFmt = 
    "<param>                                \
        <value><string>%s</string></value>  \
    </param>";                     

const char* CXmlRpcEngine::xmlRcpResponseAccountStructParamFmt = 
    "<param>                                \
        <value>                             \
            <struct>                        \
                <member>                    \
                    <name>username</name>   \
                    <value><base64>%s</base64></value> \
                </member>                   \
                <member>                    \
                    <name>password</name>   \
                    <value><base64>%s</base64></value>       \
                </member>                   \
                <member>                    \
                    <name>url</name>        \
                    <value><base64>%s</base64></value>       \
                </member>                   \
            </struct>                       \
        </value>                            \
    </param>";                     


const char* CXmlRpcEngine::xmlRpcInvalidMessageResponse = 
"<?xml version=\"1.0\"?>                                                \
<methodResponse>                                                        \
    <fault>                                                             \
        <value>                                                         \
            <struct>                                                    \
                <member>                                                \
                     <name>faultCode</name>                             \
                     <value><int>1</int></value>                        \
                </member>                                               \
                <member>                                                \
                    <name>faultString</name>                            \
                    <value><string>Invalid message format.</string></value> \
               </member>                                                \
            </struct>                                                   \
        </value>                                                        \
    </fault>                                                            \
</methodResponse>";

const char* CXmlRpcEngine::xmlRpcInvalidMethodResponse = 
"<?xml version=\"1.0\"?>                                                \
<methodResponse>                                                        \
    <fault>                                                             \
        <value>                                                         \
            <struct>                                                    \
                <member>                                                \
                     <name>faultCode</name>                             \
                     <value><int>2</int></value>                        \
                </member>                                               \
                <member>                                                \
                    <name>faultString</name>                            \
                    <value><string>Method not implemented.</string></value> \
               </member>                                                \
            </struct>                                                   \
        </value>                                                        \
    </fault>                                                            \
</methodResponse>";


const char* CXmlRpcEngine::contentLengthHdrFmt = "Content-Length: %d\r\n\r\n";

// XML-RPC methods handler
StringBuffer CXmlRpcEngine::GetUserAccountCallBack()
{
    StringBuffer xmlRpcResponse;
    ClientSettings* config = ClientSettings::getInstance();

    config->open();

    StringBuffer userName = config->getUsername();
    StringBuffer passWord = config->getPassword();
    StringBuffer serverUrl = config->getSyncURL();

    LOG.debug("%s: user name: %s", __PRETTY_FUNCTION__, userName.c_str());
    LOG.debug("%s: password: %s", __PRETTY_FUNCTION__, passWord.c_str());
    LOG.debug("%s: server URL %s", __PRETTY_FUNCTION__, serverUrl.c_str());

    StringBuffer param;

    // Encode all fields into B64 to avoid chars encoding issues
    StringBuffer userB64, pwdB64, urlB64;
    b64_encode(userB64, userName.c_str(), userName.length());
    b64_encode(pwdB64, passWord.c_str(), passWord.length());
    b64_encode(urlB64, serverUrl.c_str(), serverUrl.length());

    param.sprintf(xmlRcpResponseAccountStructParamFmt, userB64.c_str(), pwdB64.c_str(), urlB64.c_str());

    xmlRpcResponse.sprintf(xmlRpcResponseFmt, param.c_str());
 
    config->close();

    return xmlRpcResponse;
}

CXmlRpcEngine* CXmlRpcEngine::NewL(CFunambolAppUi* aAppUi)
{
    CXmlRpcEngine* self = CXmlRpcEngine::NewLC(aAppUi);
    CleanupStack::Pop(self);

    return self;
}

CXmlRpcEngine* CXmlRpcEngine::NewLC(CFunambolAppUi* aAppUi)
{
    CXmlRpcEngine* self = new (ELeave) CXmlRpcEngine(aAppUi);
    
    CleanupStack::PushL(self);
    self->ConstructL();

    return self; 
}

void CXmlRpcEngine::ConstructL()
{
    // create application session with socket server
    User::LeaveIfError(iSocketSession.Connect());

    CActiveScheduler::Add(this);
}

CXmlRpcEngine::CXmlRpcEngine(CFunambolAppUi* aAppUi) : CActive(CActive::EPriorityStandard), 
        iState(EXmlRpcMessageWait), iAppUi(aAppUi), iParserError(EFalse), iMessageParseComplete(EFalse) {}

CXmlRpcEngine::~CXmlRpcEngine()
{
    iSocket.CancelAll();
    iSocket.Close();
    iSocketSession.Close(); // this closes all sockets too

    CActive::Cancel();
}

void CXmlRpcEngine::DoCancel()
{
    iSocket.CancelAll();
    iState = EXmlRcpStopped; 
}

void CXmlRpcEngine::RunL()
{
    if ( ( iStatus == KErrDisconnected ) || ( iStatus == KErrEof ) ) {
        // disconnected, go back to reading
        LOG.debug("%s: client disconnected", __PRETTY_FUNCTION__);
        iSocket.Close();
        iSocket.Open(iSocketSession);
        iListen.Accept(iSocket, iStatus);
        iState = EXmlRpcMessageWait;
        SetActive();
    } else if (iStatus == KErrNone) {
        switch (iState) {
            case EXmlRpcMessageWait:
            {
                iSocketBuf.Zero();
                iSocket.RecvOneOrMore(iSocketBuf, 0, iStatus, iDummy);
                iState = EXmlRpcMessageParse;
                SetActive();

                break;
            }
    
            case EXmlRpcMessageParse: 
            {
                bool res = false;

                // do XML-RPC message processing 
                if (res = ParseXmlMsg()) {
                    LOG.info("XML message successfully parsed: calling callback handler...");
                    iXmlRpcResponse = iXmlRcpCallback();
                    LOG.info("rpc callback handler done: rpc message: \"%s\"", iXmlRpcResponse.c_str());
                } else {
                    LOG.error("error parsing XML-RPC message");
                }
                    
                iState = EXmlRpcSendResult;
                
                TRequestStatus* status = &iStatus;
                User::RequestComplete(status, KErrNone);
                SetActive();
                
                break;
            }
	
            case EXmlRpcSendResult: 
            {
                HBufC8* responseToClient = NULL;
                StringBuffer responseMsg;

                // add transport header with payload size
                responseMsg.sprintf(contentLengthHdrFmt, iXmlRpcResponse.length());
                responseMsg.append( iXmlRpcResponse.c_str() );

                LOG.debug("%s: sending response to client (%s)", 
                        __PRETTY_FUNCTION__, responseMsg.c_str());
                
                responseToClient = charToNewBuf8(responseMsg.c_str());

                iSocketBuf.Zero();
    
                iSocketBuf.Append( responseToClient->Des() );
                iSocket.Write(iSocketBuf, iStatus);
                
                delete responseToClient;

                iState = EXmlRpcMessageWait;
                SetActive();

                break;
            }

            default:
                break;
        }
    } else {
        LOG.error("invalid socket status");
        iStatus = KErrEof; // disconnect and restart
    }
}

void CXmlRpcEngine::StartL()
{
    LOG.debug("%s: opening bind socket", __PRETTY_FUNCTION__);
    // listen for an incoming connection
    iListen.Open(iSocketSession, KAfInet, KSockStream, KProtocolInetTcp);
   
    LOG.debug("%s: opening connection socket", __PRETTY_FUNCTION__); 
    // build the connection & transfer data
    iSocket.Open(iSocketSession);
    
    // bind the listening socket to the required port 
    TInetAddr anyAddrOnPort(KInetAddrLoop, KSyncClientPort);
    iAddress = anyAddrOnPort;
    
    if (iListen.Bind(iAddress) == KErrNone) {
        LOG.debug("%s: start listening on socket", __PRETTY_FUNCTION__);
        // everything ok
        MakeAccept();
    } else {
        LOG.error("can't bind on socket");
    }
}

void CXmlRpcEngine::Stop() 
{
    iSocket.CancelAll();
    iSocket.Close();
    iListen.Close();
}

void CXmlRpcEngine::MakeAccept(void)
{
    LOG.debug("%s: listening for mail client connection", __PRETTY_FUNCTION__);

    // Listen for incoming connections...
    iListen.Listen(KListenBackLog);

    // accept an incoming connection: subsequent data transfer will
    // occur using the socket iSocket
    iListen.Accept(iSocket, iStatus);
    LOG.debug("%s: client connection accepted", __PRETTY_FUNCTION__);
    iState = EXmlRpcMessageWait;

    SetActive();
}

void CXmlRpcEngine::MakeBind()
{
    if (iListen.Bind(iAddress)!=KErrNone) {
        LOG.error("cannot bind to local port!");
    } else {
        MakeAccept();
    }
}


bool CXmlRpcEngine::ParseXmlMsg()
{
    unsigned int pos = 0, start = 0, end = 0;
    const char *xmlMessage = NULL;
    const char *methodName = NULL;
    bool found = false;
    StringBuffer rpcMethodName;
    StringBuffer socketData = buf8ToStringBuffer(iSocketBuf);

    if (socketData.empty()) {
        LOG.error("%s: error reading client message", __PRETTY_FUNCTION__);
        iXmlRpcResponse = xmlRpcInvalidMessageResponse;
    
        return false;
    } 

    xmlMessage = XMLProcessor::copyElementContent(socketData.c_str(), "methodCall", &pos);

    if (xmlMessage == NULL) {
        LOG.error("invalid message format: \"%s\" (not a XML-RPC message)",
            socketData.c_str());
        iXmlRpcResponse = xmlRpcInvalidMessageResponse;
        return false;
    }

    if ((methodName = XMLProcessor::getElementContent(xmlMessage, "methodName", &pos, &start, &end)) == NULL) {
        LOG.error("can't find method name in xml message");
        delete [] xmlMessage;

        iXmlRpcResponse = xmlRpcInvalidMessageResponse;
        return false;
    }

    rpcMethodName.append(methodName, end - start);

    delete [] xmlMessage;

    for (int i = 0; xmlRpcHandlersTable[i].handlerName != NULL; i++) {
        LOG.info("looking up table - method name: %s",  xmlRpcHandlersTable[i].handlerName);
        if (strcmp(rpcMethodName.c_str(), xmlRpcHandlersTable[i].handlerName) == 0) {
            found = true;
            iXmlRcpCallback = xmlRpcHandlersTable[i].callBack;
            break;
        }
    }

    if (!found) {
        LOG.error("unrecognized method \"%s\" in xml-rpc message", rpcMethodName.c_str());
        iXmlRcpCallback = NULL; // reset callback handler 
        iXmlRpcResponse = xmlRpcInvalidMethodResponse;
        return false;
    }

    return true;
}
