/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "event/SymbianSyncItemListener.h"
#include "base/util/symbianUtils.h"
#include "base/adapter/PlatformAdapter.h"
#include "base/util/StringBuffer.h"
#include "base/util/stringUtils.h"
#include "base/SymbianLog.h"
#include "ClientSettings.h"
#include "SyncEngine.h"
#include "utils.h"


SymbianSyncItemListener::SymbianSyncItemListener(CSyncEngine* aSyncEngine) 
    : iSyncEngine(aSyncEngine), itemsReceived(0), itemsSent(0)
{}


void SymbianSyncItemListener::itemAddedByServer(SyncItemEvent &event)
{
    iSyncEngine->onListenerStatusUpdate(getSourceIDFromName(event.getSourceName()),
            ESyncStatusReceivingItemNumber, ++itemsReceived);
}

void SymbianSyncItemListener::itemAddedByClient(SyncItemEvent &event)
{
    TInt sourceID = getSourceIDFromName(event.getSourceName());
    if (sourceID == SOURCE_PHOTOS) {
        if (ClientSettings::getInstance()->getServerMediaHttpUpload()) {
            // Ignore this event: it's just the syncML metadata (no item content)
            return;
        }
    }
    
    iSyncEngine->onListenerStatusUpdate(sourceID, ESyncStatusSendingItemNumber, ++itemsSent);
}

void SymbianSyncItemListener::itemUpdatedByServer(SyncItemEvent &event)
{
    iSyncEngine->onListenerStatusUpdate(getSourceIDFromName(event.getSourceName()),
            ESyncStatusReceivingItemNumber, ++itemsReceived);
}

void SymbianSyncItemListener::itemUpdatedByClient(SyncItemEvent &event)
{
    TInt sourceID = getSourceIDFromName(event.getSourceName());
    if (sourceID == SOURCE_PHOTOS) {
        if (ClientSettings::getInstance()->getServerMediaHttpUpload()) {
            // Ignore this event: it's just the syncML metadata (no item content)
            return;
        }
    }
    
    iSyncEngine->onListenerStatusUpdate(sourceID, ESyncStatusSendingItemNumber, ++itemsSent);
}

void SymbianSyncItemListener::itemUploadedByClient(SyncItemEvent &event)
{
    TInt sourceID = getSourceIDFromName(event.getSourceName());
    if (sourceID == SOURCE_PHOTOS) {
        if (ClientSettings::getInstance()->getServerMediaHttpUpload()) {
            // A media item has been uploaded to the server
            iSyncEngine->onListenerStatusUpdate(sourceID, ESyncStatusSendingItemNumber, ++itemsSent);
        }
    }
}


void SymbianSyncItemListener::itemDeletedByServer(SyncItemEvent &event)
{
    StringBuffer key;
    key.convert(event.getItemKey());
    LOG.debug("item deleted by server: '%s'", key.c_str());
}

void SymbianSyncItemListener::itemDeletedByClient(SyncItemEvent &event)
{
    StringBuffer key;
    key.convert(event.getItemKey());
    LOG.debug("item delete by client: '%s'", key.c_str());
}
    
