/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <bautils.h>
#include "utils.h"
#include "commonDefs.h"
#include "base/globalsdef.h"
#include "base/FConnection.h"
#include "base/SymbianLog.h"
#include "base/util/stringUtils.h"
#include "uid3.h"
#include "customization.h"
#include "ClientSettings.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/DMTree.h"

#include <apgcli.h>  
#include <apacmdln.h> // for CApaCommandLine
#include <apgtask.h>  // apaTask
#include <eikenv.h>   // for CEikonEnv::Static() 


using namespace Funambol;


TInt getSourceIDFromName(const StringBuffer& name)
{
    if (name == SOURCE_CONTACTS_NAME) {
        return SOURCE_CONTACTS;
    }
    else if (name == SOURCE_CALENDAR_NAME) {
        return SOURCE_CALENDAR;
    }
    else if (name == SOURCE_NOTES_NAME) {
        return SOURCE_NOTES;
    }
    else if (name == SOURCE_MAIL_NAME) {
        return SOURCE_MAIL;
    }
    else if (name == SOURCE_BRIEFCASE_NAME) {
        return SOURCE_BRIEFCASE;
    }
    else if (name == SOURCE_PHOTOS_NAME) {
    	return SOURCE_PHOTOS;
    }
    else if (name == SOURCE_CONFIG_NAME) {
        return SOURCE_CONFIG;
    }
    return -1;  // Not found
}

StringBuffer getSourceNameFromID(const TInt aSourceID)
{
    StringBuffer name("");
    switch (aSourceID)
    {
        case SOURCE_CONTACTS:
            name = SOURCE_CONTACTS_NAME;
            break;
        case SOURCE_CALENDAR:
            name = SOURCE_CALENDAR_NAME;
            break;
        case SOURCE_NOTES:
            name = SOURCE_NOTES_NAME;
            break;
        case SOURCE_MAIL:
            name = SOURCE_MAIL_NAME;
            break;
        case SOURCE_BRIEFCASE:
            name = SOURCE_BRIEFCASE_NAME;
            break;
        case SOURCE_CONFIG:
            name = SOURCE_CONFIG_NAME;
            break;
        case SOURCE_PHOTOS:
        	name = SOURCE_PHOTOS_NAME;
        	break;
    }
    return name;
}


StringBuffer getSyncRunnerName(const TSyncRunners syncRunner)
{
    StringBuffer name("");
    switch (syncRunner)
    {
        case ESyncTriggeredManual:
            name = "manual";
            break;
        case ESyncTriggeredPush:
            name = "S2C push";
            break;
        case ESyncTriggeredClientPush:
            name = "C2S push";
            break;
        case ESyncTriggeredScheduled:
            name = "schedule";
            break;
    }
    return name;
}


TInt getSchedulerIndexFromMinutes(const TInt minutes)
{
    if (minutes <= 15)       { return 0; }
    else if (minutes <= 30)  { return 1; }
    else if (minutes <= 60)  { return 2; }
    else if (minutes <= 120) { return 3; }
    else if (minutes <= 240) { return 4; }
    else if (minutes <= 480) { return 5; }
    else if (minutes <= 720) { return 6; }
    else                     { return 7; }
}

TInt getSchedulerResourceIDFromMinutes(const TInt minutes)
{
    if (minutes <= 15)       { return R_STRING_15_MIN; }
    else if (minutes <= 30)  { return R_STRING_30_MIN; }
    else if (minutes <= 60)  { return R_STRING_1_HOUR; }
    else if (minutes <= 120) { return R_STRING_2_HOUR; }
    else if (minutes <= 240) { return R_STRING_4_HOUR; }
    else if (minutes <= 480) { return R_STRING_8_HOUR; }
    else if (minutes <= 720) { return R_STRING_12_HOUR; }
    else                     { return R_STRING_1_DAY; }
}

TInt getSchedulerResourceIDFromIndex(const TInt index)
{
    switch (index)
    {
        case 0:  return R_STRING_15_MIN;
        case 1:  return R_STRING_30_MIN;
        case 2:  return R_STRING_1_HOUR;
        case 3:  return R_STRING_2_HOUR;
        case 4:  return R_STRING_4_HOUR;
        case 5:  return R_STRING_8_HOUR;
        case 6:  return R_STRING_12_HOUR;
        case 7:  return R_STRING_1_DAY;
        default: return R_STRING_15_MIN;
    }
}


TInt getMinutesFromSchedulerIndex(const TInt index)
{
    switch (index)
    {
        case 0:  return 15;
        case 1:  return 30;
        case 2:  return 60;
        case 3:  return 120;
        case 4:  return 240;
        case 5:  return 480;
        case 6:  return 720;
        case 7:  return 1440;
        default: return 15;
    }
}

bool startConnection(const TInt maxNumRetries) 
{
    FConnection* connection = FConnection::getInstance();
    if (connection->isConnected() == false) {
        if (connection->startConnection()) {
            if (maxNumRetries == 0) {
                LOG.error("Connection error: 1 attempt failed.");
                return false;
            }
            LOG.debug("Error in startConnection: retry asking APN to the user.");
            int numretries = 1;
            while (connection->startConnection("")) 
            {
                LOG.debug("Error in startConnection: retry (%d) asking APN to the user.", numretries);
                if (numretries == maxNumRetries) {
                    LOG.error("Connection error: %d attempts failed.", numretries);
                    return false;
                }
                numretries ++;
            }
        }
    }
    return true;
}

bool closeConnection() {
    
    bool ret = false;
    // Close gracefully connection, if not used by other clients.
    FConnection* conn = FConnection::getInstance();
    if (conn->isConnected()) {
        conn->closeConnection();
        ret = true;
    }
    return ret;
}


void appendIcon(CArrayPtr<CGulIcon>* icons, const TInt bmpID, TInt maskID)
{
    if (!maskID) {
        maskID = bmpID + 1;     // The mask is usally the next one
    }
    CFbsBitmap *bitmap, *mask;
    
    RBuf iconsPath;
    iconsPath.Assign(charToNewBuf(LOCALISABLE_RESOURCE_FILE_MIF));
    
    // Please do not trap here...
    AknIconUtils::CreateIconLC(bitmap, mask, iconsPath, bmpID, maskID);
    icons->AppendL(CGulIcon::NewL(bitmap, mask)); 
    
    CleanupStack::Pop(2);   // bitmap, mask
    iconsPath.Close();
}


int getBuildNumberFromVersion(const char* swv) {

    int major=0, minor=0, build=0;
    if (!swv) {
        return 0;
    }
    sscanf(swv, "%d.%d.%d", &major, &minor, &build);
    return (major*10000 + minor*100 + build);
}

int mergeRArray(RArray<TInt>& destArray, const RArray<TInt>& sourceArray)
{
    for (TInt i=0; i<sourceArray.Count(); i++)
    {
        if (destArray.Find((TInt)sourceArray[i]) == KErrNotFound) {
            destArray.Append((TInt)sourceArray[i]);
        }
    }
    return destArray.Count();
}

bool removeFromRArray(const TInt itemToRemove, RArray<TInt>& array) 
{
    TBool removed = EFalse;
    for (TInt i=0; i<array.Count(); ) {
        if (array[i] == itemToRemove) {
            array.Remove(i);
            removed = ETrue;
        } 
        else i++;
    }
    return removed;
}

TBool massStorageMemoryAvailable()
{
    RFs fsSession;
    TVolumeInfo volumeInfo;
    TBool isAvailable = EFalse;

    if (fsSession.Connect() != KErrNone) {
        LOG.error("error creating session with file server");
        return isAvailable;
    }

    isAvailable = massStorageMemoryAvailable(fsSession);

    fsSession.Close();

    return isAvailable; 
}

TBool massStorageMemoryAvailable(const RFs& aFsSession)
{
    TBool isAvailable = EFalse;
    TVolumeInfo volumeInfo;

    // if mass storage is present device E: must be available
    if (aFsSession.Volume(volumeInfo, EDriveE) == KErrNone) {
        TDriveInfo driveInfo; 
        aFsSession.Drive(driveInfo, EDriveF); 

        if (driveInfo.iDriveAtt != (TUint)KDriveAbsent) {
            isAvailable = ETrue;    
        }
    }

    return isAvailable; 
}

TBool memoryCardAvailable()
{
    RFs fsSession;
    TVolumeInfo volumeInfo;
    TBool isAvailable = EFalse;

    if (fsSession.Connect() != KErrNone) {
        LOG.error("error creating session with file server");
        return isAvailable;
    }

    isAvailable = memoryCardAvailable(fsSession);

    fsSession.Close();

    return isAvailable; 
}

TBool memoryCardAvailable(const RFs& aFsSession)
{
    TBool isAvailable = EFalse;
    TVolumeInfo volumeInfo;
    TInt driveNum;

    if (massStorageMemoryAvailable(aFsSession)) {
        driveNum = EDriveF;
    } else {
        driveNum = EDriveE;
    }

    if (aFsSession.Volume(volumeInfo, driveNum) == KErrNone) {
        isAvailable = ETrue;    
    }

    return isAvailable; 
}

bool getImagesPath(TInt aStorageType, StringBuffer& imagesPath)
{
    RFs fsSession;
    TVolumeInfo volumeInfo;
    TInt driveNum = -1;
    TInt storageTypeSet = EStorageTypeUnset;
    TChar driveLetter;
    TBool isMassMemoryStorageAvailable = EFalse;
    TBool isMemoryCardAvailable = EFalse;
    bool driveAvailable = true;

    if ((aStorageType != EPhoneMemoryStorage) && (aStorageType != EMassMemoryStorage) &&
            (aStorageType != EMemoryCardStorage) && (aStorageType != EStorageTypeUnset)) {
        LOG.info("invalid storage type specified: resetting it: %d", aStorageType);
   
        aStorageType = EStorageTypeUnset; 
    }

    fsSession.Connect();

    if (massStorageMemoryAvailable(fsSession)) {
        isMassMemoryStorageAvailable = ETrue;
    } 
   
    if (memoryCardAvailable(fsSession)) {
        isMemoryCardAvailable = ETrue;
    }

    switch (aStorageType) {
        case EStorageTypeUnset: { 
            if (isMassMemoryStorageAvailable || isMemoryCardAvailable) {
                driveNum = EDriveE;
            } 

            break; 
        }
        
        case EMassMemoryStorage: {
            if (isMassMemoryStorageAvailable) {
                driveNum = EDriveE;
                storageTypeSet = EMassMemoryStorage;
            } else {
                driveAvailable = false;
            }

            break;
        }

        case EMemoryCardStorage: {
            if (isMemoryCardAvailable) {
                if (isMassMemoryStorageAvailable) {
                    driveNum = EDriveF;
                } else {
                    driveNum = EDriveE;
                }

                storageTypeSet = EMemoryCardStorage;
            } else {
                driveAvailable = false;
            }

            break;
        }
    }

    // fallback on main phone memory
    if (driveNum == -1) {
        driveNum = EDriveC;
        storageTypeSet = EPhoneMemoryStorage;
    }

    fsSession.DriveToChar(driveNum, driveLetter);
    StringBuffer picturesPath = bufToStringBuffer(PathInfo::ImagesPath());

    if (storageTypeSet == EPhoneMemoryStorage) {
        // main phone memory has a specific user writable folder (usually /data/)
        StringBuffer rootPath = bufToStringBuffer(PathInfo::PhoneMemoryRootPath());
        imagesPath.sprintf("%s%s", rootPath.c_str(), picturesPath.c_str());
    } else {
        imagesPath.sprintf("%c:\\%s", TUint(driveLetter), picturesPath.c_str());
    }

    fsSession.Close();

    LOG.debug("image path: %s", imagesPath.c_str());
    
    return driveAvailable;
}

const StringBuffer getDriveName(TInt aStorageType)
{
    StringBuffer driveName;
    RFs fsSession;
    TInt driveNum = -1;
    TChar driveLetter;

    if (fsSession.Connect() != KErrNone) {
        LOG.error("error connecting to file server");

        return NULL;
    }

    switch (aStorageType) {
        case EPhoneMemoryStorage:
            driveNum = EDriveC;
            break;
        case EMassMemoryStorage:
            driveNum = EDriveE;
            break;
        case EMemoryCardStorage: {
            if (massStorageMemoryAvailable(fsSession)) {
                driveNum = EDriveF;
            } else {
                driveNum = EDriveE;
            }
           
            break; 
        }
        case EStorageTypeUnset: 
        default: 
            LOG.error("%s: invalid storage type specified", __FUNCTION__);
            break;
    }

    if (driveNum > 0) {
        fsSession.DriveToChar(driveNum, driveLetter);
        driveName.sprintf("%c", TUint(driveLetter));
    }

    fsSession.Close();

    return (driveName.empty() ? NULL : driveName.c_str());
}

StringBuffer getSourceNameFromServerURI(const StringBuffer& uri)
{
    StringBuffer sourceName, nodeName;
    
    //
    // Access the config nodes for sources
    //
    nodeName.sprintf("%s/%s", APPLICATION_URI, CONTEXT_SPDS_SOURCES);
    DMTree* dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
    ManagementNode *node = dmt->readManagementNode(nodeName.c_str());
    if (!node) {
        setErrorF(ERR_INVALID_CONTEXT, ERRMSG_INVALID_CONTEXT, nodeName.c_str());
        LOG.error("getSourceIDFromServerURI: error: %s", getLastErrorMsg());
        goto finally;
    }
    
    //
    // Find the source name from its serverURI.
    //
    for(int i=0; i<node->getChildrenMaxCount(); i++) {
        ManagementNode *src = node->getChild(i);
        char *tmp = src->readPropertyValue(PROPERTY_SOURCE_URI);
        //LOG.debug("i:%d uri:%s tmp:%s", i, uri, tmp);
        if (tmp && !strcmp(uri.c_str(), tmp)) {
            char* name = src->readPropertyValue(PROPERTY_SOURCE_NAME);
            sourceName = name;
            delete [] name;
            delete [] tmp;
            break;
        }
        delete [] tmp;
    }
    

    /*
    * If the client has modified the source name, in the push the server continue
    * to send i.e. scard for the contact, scal for the calendar and stask for the task.
    * mail for the mail. So if the previous loop doesn't find anything, try in this way
    */
    if (sourceName.length() == 0) 
    {
        if (uri == "mail") {
            sourceName = SOURCE_MAIL_NAME;
        }
        else if ((uri == "scard")  || (uri == "card") || (uri == "oxcard")) {
            sourceName = SOURCE_CONTACTS_NAME;
        }
        else if ( (uri == "scal")  || (uri == "cal")  || (uri == "event") ||
                  (uri == "stask") || (uri == "task") || 
                  (uri == "oxcal") || (uri == "oxtask") ) {
            sourceName = SOURCE_CALENDAR_NAME;
        } 
        else if ((uri == "snote")  || (uri == "note")) {
            sourceName = SOURCE_NOTES_NAME;
        }
    }

finally:
    delete dmt;
    delete node;
    return sourceName;
}

void StartServerProcessL(const TDesC& aServerName)
{
    TThreadId app_threadid;
    CApaCommandLine* cmdLine; 
    RApaLsSession ls;
    TInt err = KErrNone;
    TBool ret(ETrue);

    cmdLine = CApaCommandLine::NewLC();
    cmdLine->SetExecutableNameL( aServerName );
    cmdLine->SetCommandL( EApaCommandRun );

    User::LeaveIfError(ls.Connect());
    if (err = ls.StartApp(*cmdLine, app_threadid)) {
        ret = EFalse;            
    }

    ls.Close();

    CleanupStack::PopAndDestroy(); // cmdLine
}

void StartProcessL(const TDesC& aExe, const TDesC& aCmdLine)
{
    RProcess proc;
    User::LeaveIfError(proc.Create(aExe, aCmdLine));
    proc.Resume();
    proc.Close();
}

void StartViewProcessL(const TUid& aUid, const TDesC& aDocument)
{
    TThreadId app_threadid;
    RApaLsSession ls;
    User::LeaveIfError(ls.Connect());
    ls.StartDocument(aDocument, aUid, app_threadid);
    ls.Close();
}

TBool CheckProcessRunning(const TUid& aProcessUid)
{
    TBool ret(EFalse);
    TFileName res;
    TFindProcess find;

    while (find.Next(res) == KErrNone) {
        RProcess ph;
        ph.Open(res);

        // check process UID
        if (ph.Identity() == aProcessUid) {
            ret = ETrue;
            ph.Close();
            break;
        }

        ph.Close();
    }

    return ret;
}

void openAppL(const TUid& aAppUid, const TDesC& aDocument)
{
    TUid appUid(aAppUid); 
    RApaLsSession apaLsSession;
    TApaTaskList taskList(CEikonEnv::Static()->WsSession());
    TApaTask task = taskList.FindApp(appUid);

    if(task.Exists()) {
        task.BringToForeground();
    } else {
        if (!apaLsSession.Handle()) {
            User::LeaveIfError(apaLsSession.Connect());
        }

        TThreadId threadId;
        User::LeaveIfError(apaLsSession.StartDocument( aDocument, appUid, threadId ));
        apaLsSession.Close();
    }
}

TBool startMidletL(const TDesC& aMidletName)
{
    TUid midletUid = TUid::Null();
    TApaAppInfo appInfo;
    RApaLsSession apaSession;

    User::LeaveIfError( apaSession.Connect() );
    CleanupClosePushL( apaSession );
    User::LeaveIfError( apaSession.GetAllApps() );

    while ( apaSession.GetNextApp( appInfo ) == KErrNone )
    {
        if(appInfo.iFullName.Right(8).Compare(_L(".fakeapp")) == 0)
        {
            // Caption of the MIDlet can be read from appInfo.iCaption
            // When the MIDlet to be launch is found, store its UID
            // (appInfo.iUid) for later use.
            if (appInfo.iCaption.Compare(aMidletName) == 0)
            {
                midletUid = appInfo.iUid;
                break;
            }
        }
    }

    CleanupStack::PopAndDestroy(); // close apaSession

    if (midletUid == TUid::Null()) { // midlet not found
        LOG.error("startMidletL: can't find midlet in application registry");
        return EFalse; 
    } else {
        openAppL( midletUid, _L("") );
    }

    return ETrue;
}

TUint get64bitValue(const StringBuffer& stringValue)
{
    if (stringValue.empty()) {
        return 0;
    }
    
    // This piece of code is rather odd, but it looks like there are some
    // problems converting  between 32 and 64 bits value. For the timestamp we
    // don't really need to manipulate 64 bits values. We just need to store a
    // value at the end of the sync and to restore it. So we access the two
    // subwords by castint to 32bits.
    TUint ret = 0;
    TUint low = 0, high = 0;
    if (stringValue.length() > 8) {
        StringBuffer sb  = stringValue.substr(0, 8);
        StringBuffer sb1;
        sb1.sprintf("0x%s", sb.c_str());
        low = strtol(sb1.c_str(), NULL, 16);
        sb = stringValue.substr(8, 8);
        sb1.sprintf("0x%s", sb.c_str());
        high = strtol(sb1.c_str(), NULL, 16);
    } else {
        // It fits in 8 bytes
        low = atoi(stringValue.c_str());
    }
    
    TUint* p = (TUint*)&ret;
    *p     = low;
    *(p+1) = high;

    return ret;
}
