/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include <e32base.h>
#include <e32std.h>
//#include <e32cons.h>            // Console

#include "base/util/stringUtils.h"
#include "SyncProfileManager.h"
#include "Cleaner.h"
#include "customization.h"

#ifdef FAKE_IMAP_ACCOUNT
#include "FakeImapAccountManager.h"
#else
#include "ImapAccountManager.h"
#endif


using namespace Funambol;


GLDEF_C void CleanProfilesL()
{
    LOG.debug("Removing native sync profile");
    // delete sync profile
    CSyncProfileManager* profileManager;
    profileManager = CSyncProfileManager::createNewInstance();
    TBool result = profileManager->deleteProfile();
    delete profileManager;
    profileManager=NULL;
    if (result) {
        LOG.debug("OK");
    } else {
        LOG.debug("Failed");
    }
    
    if (ENABLE_SOURCE_MAIL)
    {
        // delete imap profile
        CImapAccountManager* imapManager;
        const StringBuffer profileSb(SYNC_PROFILE_NAME);
        HBufC* profile = stringBufferToNewBuf(profileSb);
        CleanupStack::PushL(profile);
    
        LOG.debug("Removig email account");
        TRAPD(err,
                imapManager = CImapAccountManager::NewL();
                imapManager->DeleteAccountL(*profile);
        );
        delete imapManager;
        imapManager = NULL;
        CleanupStack::PopAndDestroy(profile);
        LOG.debug("Done");
    }
    
    
    // Remove the log file
    RFs fsSession;
    fsSession.Connect();
    
    RBuf name;
    StringBuffer logName(LOG_FILE_PATH);
    logName += LOG_FILE_NAME;
    name.Assign(stringBufferToNewBuf(logName));
    
    fsSession.Delete(name);
    fsSession.Close();
    name.Close();
}

