/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <stdio.h>
#include "utils.h"
#include "cppunit/extensions/FCppUnit.h"

namespace CppUnit {
    TestList *TestList::instance = NULL;
    TestReport* TestReport::instance = NULL;

    void TestFixture::run() {

        setUp();
        execTests();
        tearDown();
    }

    void cppUnitAssert(bool cond) {
        if (!cond) {
            TestReport* report = TestReport::getInstance();
            report->addFailure();
        }
    }

    void cppUnitAssert(const char* msg, bool cond) {
        if (!cond) {
            TestReport* report = TestReport::getInstance();
            report->addFailure(msg);
        }
    }

    TestFixture* TestList::getNext() {
        if (iterator >= numTests) {
            return NULL;
        }
        return tests[iterator++];
    }

    void TestList::add(TestFixture* test) {
        // TODO check size...
        tests[numTests++] = test;
    }

    TestList* TestList::getInstance() {
        if (instance == NULL) {
            instance = new TestList();
        }
        return instance;
    }

    TestReport* TestReport::getInstance() {
        if (instance == NULL) {
            instance = new TestReport();
        }
        return instance;
    }

    TestReport::TestReport() : fp(NULL), filename(NULL) {
    }

    void TestReport::beginTest(const char* testName) {
        startPrinting();   
        char string[256];
        sprintf(string, "Starting test %s \n", testName);
        printMessage(string);        
        endPrinting();
    }

    void TestReport::addFailure() {
        startPrinting();
        printMessage("**** Failed\n");
        endPrinting();
    }

    void TestReport::addFailure(const char* msg) {
        startPrinting();
        char string[1024] = {0};
        strcpy(string, "**** Failed: ");
        strncat(string, msg, 1000);
        strcat(string, "\n");
        printMessage(string);
        endPrinting();
    }

    void TestReport::endTest() {
        startPrinting();
        printMessage("Completed\n");
        endPrinting();
    }

    bool TestReport::setFilename(const char* p) {
        size_t l = strlen(p);
        if (filename) {
            delete [] filename;
        }
        if (l > 0) {
            filename = new char[l + 1];
            strncpy(filename, p, l);
            return true;
        } 
        return false;

    }

    void TestReport::startPrinting() {
        if (filename) {
            fp = fopen(filename, "a+");
        } else {
            fp = fopen("c:\\data\\unittest.txt", "a+");
        }
        
    }

    void TestReport::endPrinting() {
        if (fp) {
            fclose(fp);
        }
    }

    void TestReport::printMessage(const char* msg) {
        if (fp) {
            char* time = getTime();
            fprintf(fp, time); fprintf(fp, ": ");        
            fprintf(fp, msg);
            delete [] time;
        }
    }

    void TestManager::run() {
        TestList* list = TestList::getInstance();
        TestFixture* test = list->getNext();
        while (test) {
            test->run();
            test = list->getNext();
        }
    }
}

