/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef SYMBIANCPPUNIT_INCL
#define SYMBIANCPPUNIT_INCL

#include <stdio.h>

#define CPPUNIT_STRINGIFY(x) #x

#define CPPUNIT_TEST_SUITE(name)                                \
                                 public:                        \
                                     void execTests() {         \
                                         int missingSemicolon

#define CPPUNIT_TEST(name) {                                                   \
                               CppUnit::TestReport* report = CppUnit::TestReport::getInstance(); \
                               report->beginTest(CPPUNIT_STRINGIFY(name));     \
                               name();                                         \
                               report->endTest();                              \
                           }

#define CPPUNIT_TEST_SUITE_END() }
#define CPPUNIT_ASSERT_EQUAL(val1, val2) CppUnit::cppUnitAssert(val1 == val2)
#define CPPUNIT_ASSERT(cond) CppUnit::cppUnitAssert(cond)
#define CPPUNIT_TEST_SUITE_REGISTRATION( name ) namespace CppUnit { \
                                                    name varName##name;   \
                                                }

#define CPPUNIT_ASSERT_MESSAGE(msg, cond) CppUnit::cppUnitAssert(msg, cond)

namespace CppUnit {

    class TestFixture;

    class TestList {
    public:
        TestList() : numTests(0),
                     iterator(0) {
        }
        static TestList* getInstance();
        void add(TestFixture* testFixture);
        TestFixture* getNext();
    private:
        TestFixture* tests[1024];
        int numTests;
        int iterator;
        static TestList* instance;
    };

    class TestFixture {
    public:

        TestFixture() {
            TestList* list = TestList::getInstance();
            list->add(this);
        }

        void run();

        virtual void setUp() {};
        virtual void tearDown() {};
        virtual void execTests() = 0;
    };

    class TestReport {
    public:
        static TestReport* getInstance();

        TestReport();
        ~TestReport() { if (filename) delete [] filename; filename = NULL; }
        void beginTest(const char* testName);
        void addFailure();
        void addFailure(const char* msg);
        void endTest();
        /**
        * Set the filename to write the log. Return false if the filename is empty or null
        * @param p - the filename with the whole path
        */
        bool setFilename(const char* p);

    private:
        void TestReport::startPrinting();
        void TestReport::endPrinting();
        void TestReport::printMessage(const char* msg);
    private:
        static TestReport* instance;
        FILE *fp;
        char* filename;

    };


    class TestManager {
    public:
        TestManager() {}

        void run();
    };

    void cppUnitAssert(bool cond);
    void cppUnitAssert(const char* msg, bool cond);
}

#endif
