/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 - 2009 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "utils.h"
#include "commonDefs.h"
#include "base/globalsdef.h"
#include "base/util/stringUtils.h"
#include "customization.h"
#include "base/util/utils.h"
#include "base/util/StringBuffer.h"

#include "extensions/TestFactoryRegistry.h"
#include "extensions/HelperMacros.h"

USE_NAMESPACE

class UtilsTest : public CppUnit::TestFixture
{
    CPPUNIT_TEST_SUITE(UtilsTest);
    CPPUNIT_TEST(testSourceIDFromName);
    CPPUNIT_TEST(testSourceNameFromID);
    CPPUNIT_TEST(testGetImagesPath);
    CPPUNIT_TEST(testFileOperations);
    CPPUNIT_TEST_SUITE_END();

private:
    void testSourceIDFromName() {
        CPPUNIT_ASSERT(getSourceIDFromName(SOURCE_CONTACTS_NAME) == SOURCE_CONTACTS);
        CPPUNIT_ASSERT(getSourceIDFromName(SOURCE_CALENDAR_NAME) == SOURCE_CALENDAR);
        CPPUNIT_ASSERT(getSourceIDFromName(SOURCE_NOTES_NAME) == SOURCE_NOTES);
        CPPUNIT_ASSERT(getSourceIDFromName(SOURCE_BRIEFCASE_NAME) == SOURCE_BRIEFCASE);
        CPPUNIT_ASSERT(getSourceIDFromName(SOURCE_CONFIG_NAME) == SOURCE_CONFIG);
    }

    void testSourceNameFromID() {
        CPPUNIT_ASSERT(getSourceNameFromID(SOURCE_CONTACTS) == SOURCE_CONTACTS_NAME);
        CPPUNIT_ASSERT(getSourceNameFromID(SOURCE_CALENDAR) == SOURCE_CALENDAR_NAME);
        CPPUNIT_ASSERT(getSourceNameFromID(SOURCE_NOTES) == SOURCE_NOTES_NAME);
        CPPUNIT_ASSERT(getSourceNameFromID(SOURCE_BRIEFCASE) == SOURCE_BRIEFCASE_NAME);
        CPPUNIT_ASSERT(getSourceNameFromID(SOURCE_CONFIG) == SOURCE_CONFIG_NAME);
    }

    void testGetImagesPath() {

        StringBuffer imagesPathC("C:\\Data\\Images\\");
        StringBuffer imagesPathE("E:\\Images\\");
        StringBuffer path;

        getImagesPath("C", path);       CPPUNIT_ASSERT(path == imagesPathC);
        getImagesPath("c", path);       CPPUNIT_ASSERT(path == imagesPathC);
        getImagesPath("C:", path);      CPPUNIT_ASSERT(path == imagesPathC);
        getImagesPath("C:\\", path);    CPPUNIT_ASSERT(path == imagesPathC);

        getImagesPath("E", path);       CPPUNIT_ASSERT(path == imagesPathE);
        getImagesPath("e", path);       CPPUNIT_ASSERT(path == imagesPathE);
        getImagesPath("E:", path);      CPPUNIT_ASSERT(path == imagesPathE);
        getImagesPath("E:\\", path);    CPPUNIT_ASSERT(path == imagesPathE);

        getImagesPath("", path);              CPPUNIT_ASSERT(path == imagesPathC);
        getImagesPath(" ", path);             CPPUNIT_ASSERT(path == imagesPathC);
        getImagesPath("hi", path);            CPPUNIT_ASSERT(path == imagesPathC);
        getImagesPath("hello", path);         CPPUNIT_ASSERT(path == imagesPathE);
    }

    void testFileOperations() {
        StringBuffer testFolderPath = "C:\\Data\\testFolder";
        StringBuffer testFileName = "testFile.txt";
        StringBuffer testFilePath = testFolderPath + "\\" + testFileName;
        const char* msg = "test string\n";
        size_t len = strlen(msg);
        bool ret = false;

        ret = createFolder(testFolderPath);
        CPPUNIT_ASSERT(ret == 0);

        ret = fileExists(testFilePath.c_str());
        CPPUNIT_ASSERT(ret == false);
        // create file
        ret = saveFile(testFilePath.c_str(), msg, len, false);
        CPPUNIT_ASSERT(ret == true);
        // check
        ret = fileExists(testFilePath.c_str());
        CPPUNIT_ASSERT(ret == true);

        // check reading
        char *buf = NULL;

        ret = readFile(testFilePath.c_str(), &buf, &len, false);
        CPPUNIT_ASSERT(ret == true);
        if (buf) {
            CPPUNIT_ASSERT(strcmp(msg, buf) == 0);
        }

        delete [] buf;

        // check directory reading
        int filesCount = 0;
        readDir(testFolderPath.c_str(), &filesCount, true);
        CPPUNIT_ASSERT(filesCount == 1);

        ret = removeFileInDir(testFolderPath.c_str(), testFileName.c_str());
        CPPUNIT_ASSERT(ret == true);

        ret = removeDir(testFolderPath.c_str());
        CPPUNIT_ASSERT(ret == true);
    }
};

CPPUNIT_TEST_SUITE_REGISTRATION( UtilsTest );
